@@ -6,9 +6,12 @@ package mozilla.telemetry.glean.scheduler
 
 import androidx.test.ext.junit.runners.AndroidJUnit4
 import mozilla.telemetry.glean.Glean
-import mozilla.telemetry.glean.getPlainBody
-import mozilla.telemetry.glean.getContextWithMockedInfo
+import mozilla.telemetry.glean.delayMetricsPing
+import mozilla.telemetry.glean.getContext
 import mozilla.telemetry.glean.getMockWebServer
+import mozilla.telemetry.glean.getPlainBody
+import mozilla.telemetry.glean.getWorkerStatus
+import mozilla.telemetry.glean.private.CommonMetricData
 import mozilla.telemetry.glean.private.EventMetricType
 import mozilla.telemetry.glean.private.Lifetime
 import mozilla.telemetry.glean.private.NoExtraKeys
@@ -19,15 +22,15 @@ import mozilla.telemetry.glean.resetGlean
 import mozilla.telemetry.glean.testing.GleanTestRule
 import mozilla.telemetry.glean.triggerWorkManager
 import mozilla.telemetry.glean.utils.tryGetLong
-import mozilla.telemetry.glean.delayMetricsPing
 import okhttp3.mockwebserver.MockWebServer
+import org.json.JSONObject
+import org.junit.After
 import org.junit.Assert.assertEquals
+import org.junit.Assert.assertFalse
+import org.junit.Before
 import org.junit.Rule
 import org.junit.Test
 import org.junit.runner.RunWith
-import org.junit.After
-import org.junit.Before
-import org.json.JSONObject
 import java.util.concurrent.TimeUnit
 
 /**
@@ -40,7 +43,7 @@ import java.util.concurrent.TimeUnit
  */
 @RunWith(AndroidJUnit4::class)
 class CustomPingTest {
-    private val context = getContextWithMockedInfo()
+    private val context = getContext()
     private lateinit var server: MockWebServer
 
     @get:Rule
@@ -61,9 +64,13 @@ class CustomPingTest {
         // a smoke test for custom pings
 
         delayMetricsPing(context)
-        resetGlean(context, Glean.configuration.copy(
-            serverEndpoint = "http://" + server.hostName + ":" + server.port
-        ), clearStores = true, uploadEnabled = true)
+        resetGlean(
+            context,
+            Glean.configuration.copy(
+                serverEndpoint = "http://" + server.hostName + ":" + server.port
+            ),
+            clearStores = true, uploadEnabled = true
+        )
 
         // Define a new custom ping inline.
         val customPing = PingType<NoReasonCodes>(
@@ -84,9 +91,13 @@ class CustomPingTest {
     @Test
     fun `multiple pings in one go`() {
         delayMetricsPing(context)
-        resetGlean(context, Glean.configuration.copy(
-            serverEndpoint = "http://" + server.hostName + ":" + server.port
-        ), clearStores = true, uploadEnabled = true)
+        resetGlean(
+            context,
+            Glean.configuration.copy(
+                serverEndpoint = "http://" + server.hostName + ":" + server.port
+            ),
+            clearStores = true, uploadEnabled = true
+        )
 
         // Define a new custom ping inline.
         val customPing = PingType<NoReasonCodes>(
@@ -126,28 +137,32 @@ class CustomPingTest {
         assertEquals(1L, pingInfo.tryGetLong("seq")!!)
     }
 
-    // Suppressing our own deprecation before we move over to the new event recording API.
     @Test
-    @Suppress("DEPRECATION")
-    fun `events for custom pings are flushed at startup`() {
+    fun `events for custom pings aren't flushed at startup`() {
         delayMetricsPing(context)
-        resetGlean(context, Glean.configuration.copy(
-            serverEndpoint = "http://" + server.hostName + ":" + server.port
-        ), clearStores = true, uploadEnabled = true)
+        resetGlean(
+            context,
+            Glean.configuration.copy(
+                serverEndpoint = "http://" + server.hostName + ":" + server.port
+            ),
+            clearStores = true, uploadEnabled = true
+        )
 
         val pingName = "custom-events-1"
 
         // Define a 'click' event
         val click = EventMetricType<NoExtraKeys, NoExtras>(
-            disabled = false,
-            category = "ui",
-            lifetime = Lifetime.Ping,
-            name = "click",
-            sendInPings = listOf(pingName),
-            allowedExtraKeys = listOf()
+            CommonMetricData(
+                disabled = false,
+                category = "ui",
+                lifetime = Lifetime.PING,
+                name = "click",
+                sendInPings = listOf(pingName),
+            ),
+            allowedExtraKeys = emptyList()
         )
         // and record it in the currently initialized Glean instance.
-        click.record()
+        click.record(NoExtras())
 
         // We need to simulate that the app is shutdown and all resources are freed.
         Glean.testDestroyGleanHandle()
@@ -164,15 +179,24 @@ class CustomPingTest {
         )
 
         // This is equivalent to a consumer calling `Glean.initialize` at startup
-        resetGlean(context, Glean.configuration.copy(
-            serverEndpoint = "http://" + server.hostName + ":" + server.port
-        ), clearStores = true, uploadEnabled = true)
+        resetGlean(
+            context,
+            Glean.configuration.copy(
+                serverEndpoint = "http://" + server.hostName + ":" + server.port
+            ),
+            clearStores = false, uploadEnabled = true
+        )
 
+        // There should be no ping upload worker,
+        // because there is no ping to upload.
+        assertFalse(getWorkerStatus(context, PingUploadWorker.PING_WORKER_TAG).isEnqueued)
+
+        // But if the custom ping is specifically submitted,
+        // it should be received.
+        customPing.submit()
         // Trigger work manager once.
         // This should launch one worker that handles all pending pings.
         triggerWorkManager(context)
-
-        // Receive the custom events ping.
         var request = server.takeRequest(2L, TimeUnit.SECONDS)!!
         var docType = request.path!!.split("/")[3]
         assertEquals(pingName, docType)
