@@ -3,24 +3,18 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 @testable import Glean
-import OHHTTPStubs
 import XCTest
 
-// swiftlint:disable function_body_length
 class MetricsPingSchedulerTests: XCTestCase {
     var expectation: XCTestExpectation?
 
-    override func setUp() {
-        Glean.shared.enableTestingMode()
-    }
-
     override func tearDown() {
         expectation = nil
-        OHHTTPStubs.removeAllStubs()
+        tearDownStubs()
     }
 
     func testIsAfterDueTime() {
-        let mps = MetricsPingScheduler()
+        let mps = MetricsPingScheduler(true)
         var fakeNow = DateComponents()
         var fakeDate = Date()
 
@@ -74,7 +68,7 @@ class MetricsPingSchedulerTests: XCTestCase {
     }
 
     func testGetLastCollectedDate() {
-        let mps = MetricsPingScheduler()
+        let mps = MetricsPingScheduler(true)
 
         // getLastCollectedDate must report nil when no stored date is available
         UserDefaults.standard.set(nil, forKey: MetricsPingScheduler.Constants.lastMetricsPingSentDateTime)
@@ -108,13 +102,12 @@ class MetricsPingSchedulerTests: XCTestCase {
     }
 
     func testSchedulePingCollection() {
-        let mps = MetricsPingScheduler()
+        let mps = MetricsPingScheduler(true)
         let now = Date()
 
-        // Here we reset Glean and set uploadEnabled to false to ensure that this does not generate a
-        // metrics ping. We are only interested in the updated date that is scheduled after the call
-        // to `collectPingAndReschedule`.
-        Glean.shared.resetGlean(configuration: Configuration(), clearStores: true, uploadEnabled: false)
+        // Here we reset Glean and just discard any pings that might be generated since we are only
+        // interested in the updated date that is scheduled after the call to `collectPingAndReschedule`.
+        resetGleanDiscardingInitialPings(testCase: self, tag: "MetricsPingSchedulerTests")
 
         UserDefaults.standard.set(nil, forKey: MetricsPingScheduler.Constants.lastMetricsPingSentDateTime)
         mps.collectPingAndReschedule(now, reason: GleanMetrics.Pings.MetricsReasonCodes.overdue)
@@ -141,52 +134,54 @@ class MetricsPingSchedulerTests: XCTestCase {
             mps.timer?.fireDate.toISO8601String(precision: .second),
             "schedulePingCollection must schedule next collection on the next day"
         )
+
+        Glean.shared.testDestroyGleanHandle()
     }
 
     func testQueuedDataNotInOverdueMetricsPings() {
         // Reset Glean and do not start it right away
         Glean.shared.testDestroyGleanHandle()
-        Dispatchers.shared.setTaskQueueing(enabled: true)
+        Glean.shared.enableTestingMode()
 
         // Set the last time the "metrics" ping was sent to now. This is required for us to not
         // send a metrics pings the first time we initialize Glean.
         let now = Date()
-        Glean.shared.metricsPingScheduler.updateSentDate(now)
+        MetricsPingScheduler(true).updateSentDate(now)
 
         // Create a metric and set its value. We expect this to be sent in the first ping
         // that gets generated the SECOND time we start Glean.
-        let expectedStringMetric = StringMetricType(
+        let expectedStringMetric = StringMetricType(CommonMetricData(
             category: "telemetry",
             name: "expected_metric",
             sendInPings: ["metrics"],
             lifetime: Lifetime.ping,
             disabled: false
-        )
+        ))
         let expectedValue = "must-exist-in-the-first-ping"
 
-        Glean.shared.resetGlean(clearStores: false)
+        resetGleanDiscardingInitialPings(testCase: self, tag: "MetricsPingSchedulerTests", clearStores: false)
+
         expectedStringMetric.set(expectedValue)
 
         // Destroy Glean, it will retain the recorded metric.
-        Glean.shared.testDestroyGleanHandle()
-        Dispatchers.shared.setTaskQueueing(enabled: true)
+        Glean.shared.testDestroyGleanHandle(false)
 
         // Create data and attempt to record data before Glean is initialized.  This will
         // be queued in the dispatcher.
-        let stringMetric = StringMetricType(
+        let stringMetric = StringMetricType(CommonMetricData(
             category: "telemetry",
             name: "canary_metric",
             sendInPings: ["metrics"],
             lifetime: Lifetime.ping,
             disabled: false
-        )
+        ))
         let canaryValue = "must-not-be-in-the-first-ping"
         stringMetric.set(canaryValue)
 
         // Set the last time the "metrics" ping was sent to yesterday, which should make
         // the ping overdue and trigger it at startup.
         let yesterday = Calendar.current.date(byAdding: Calendar.Component.day, value: -1, to: now)
-        Glean.shared.metricsPingScheduler.updateSentDate(yesterday!)
+        MetricsPingScheduler(true).updateSentDate(yesterday!)
 
         stubServerReceive { pingType, json in
             if pingType != "metrics" {
@@ -218,45 +213,49 @@ class MetricsPingSchedulerTests: XCTestCase {
         // the previous run) but must not send the canary string, which would be sent the next time
         // the "metrics" ping is collected after this one.
         // Glean.shared.initialize(uploadEnabled: true)
-        Glean.shared.initialize(uploadEnabled: true)
+        Glean.shared.enableTestingMode()
+        Glean.shared.setLogPings(true)
+        Glean.shared.initialize(uploadEnabled: true, buildInfo: stubBuildInfo())
+        // Enable ping logging for all tests
         waitForExpectations(timeout: 5.0) { error in
             XCTAssertNil(error, "Test timed out waiting for upload: \(error!)")
         }
 
         // Clean up
-        Glean.shared.resetGlean(clearStores: true)
+        resetGleanDiscardingInitialPings(testCase: self, tag: "MetricsPingSchedulerTests")
         Glean.shared.testDestroyGleanHandle()
     }
 
     func testGleanPreservesLifetimeApplicationMetrics() {
         // Reset Glean and do not start it right away
         Glean.shared.testDestroyGleanHandle()
-        Dispatchers.shared.setTaskQueueing(enabled: true)
+        Glean.shared.enableTestingMode()
 
         // Set the last time the "metrics" ping was sent to now. This is required for us to not
         // send a metrics pings the first time we initialize Glean.
         let now = Date()
-        Glean.shared.metricsPingScheduler.updateSentDate(now)
+        MetricsPingScheduler(true).updateSentDate(now)
 
         // Create a metric and set its value. We expect this to be sent in the first ping
         // that gets generated the SECOND time we start Glean.
-        let testMetric = StringMetricType(
+        let testMetric = StringMetricType(CommonMetricData(
             category: "telemetry",
             name: "test_applifetime_metric",
             sendInPings: ["metrics"],
             lifetime: Lifetime.application,
             disabled: false
-        )
+        ))
         let expectedValue = "I-will-survive!"
 
         // Reset Glean and start it for the FIRST time, then record a value.
-        Glean.shared.resetGlean(clearStores: false)
+        resetGleanDiscardingInitialPings(testCase: self, tag: "MetricsPingSchedulerTests", clearStores: false)
+
         testMetric.set(expectedValue)
 
         // Set the last time the "metrics" ping was sent to yesterday, which should make
         // the ping overdue and trigger it at startup.
         let yesterday = Calendar.current.date(byAdding: Calendar.Component.day, value: -1, to: now)
-        Glean.shared.metricsPingScheduler.updateSentDate(yesterday!)
+        Glean.shared.metricsPingScheduler!.updateSentDate(yesterday!)
 
         // Set up the interception of the ping for inspection
         stubServerReceive { pingType, json in
@@ -291,10 +290,10 @@ class MetricsPingSchedulerTests: XCTestCase {
 
         // Checking to see if the metric has been cleared after startup here, since receiving
         // the response above means that the metric should have been processed and cleared.
-        XCTAssertFalse(testMetric.testHasValue(), "The metric must be cleared after startup")
+        XCTAssertNil(testMetric.testGetValue(), "The metric must be cleared after startup")
 
         // Clean up
-        Glean.shared.resetGlean(clearStores: true)
+        resetGleanDiscardingInitialPings(testCase: self, tag: "MetricsPingSchedulerTests")
         Glean.shared.testDestroyGleanHandle()
     }
 
@@ -305,6 +304,7 @@ class MetricsPingSchedulerTests: XCTestCase {
 
         init(expectation: XCTestExpectation?) {
             mpsExpectation = expectation
+            super.init(true)
         }
 
         override func collectPingAndReschedule(
@@ -317,12 +317,12 @@ class MetricsPingSchedulerTests: XCTestCase {
     }
 
     func testTimerInvocation() {
+        // Reset Glean clearing the stores so that the metrics ping will be empty
+        resetGleanDiscardingInitialPings(testCase: self, tag: "MetricsPingSchedulerTests")
+
         // Set up the expectation
         expectation = expectation(description: "Timer fired")
 
-        // Reset Glean clearing the stores so that the metrics ping will be empty
-        Glean.shared.resetGlean(clearStores: true)
-
         // Build the mock MPS passing in the expectation to be fulfilled later
         let mps = FakeMPS(expectation: expectation)
 
