@@ -5,7 +5,7 @@
 """The setup script."""
 
 # setuptools must be imported first
-from setuptools import setup, Distribution, find_packages
+from setuptools import setup, Distribution
 from setuptools.command.install import install
 
 from distutils.command.build import build as _build
@@ -56,16 +56,13 @@ with (SRC_ROOT / "CHANGELOG.md").open() as history_file:
     history = history_file.read()
 
 # glean version. Automatically updated by the bin/prepare_release.sh script
-version = "39.1.0"
+version = "52.0.0"
 
 requirements = [
-    "cffi>=1.13.0",
-    "glean_parser==3.6.0",
-    "iso8601>=0.1.10; python_version<='3.6'",
+    "semver>=2.13.0",
+    "glean_parser~=6.4",
 ]
 
-setup_requirements = ["cffi>=1.13.0"]
-
 # The environment variable `GLEAN_BUILD_VARIANT` can be set to `debug` or `release`
 buildvariant = os.environ.get("GLEAN_BUILD_VARIANT", "debug")
 
@@ -183,11 +180,9 @@ class build(_build):
             "cargo",
             "build",
             "--package",
-            "glean-ffi",
+            "glean-bundle",
             "--target",
             target,
-            "--features",
-            "rkv-safe-mode",
         ]
         if buildvariant != "debug":
             command.append(f"--{buildvariant}")
@@ -195,12 +190,27 @@ class build(_build):
         if "-darwin" in target:
             env["MACOSX_DEPLOYMENT_TARGET"] = macos_compat(target)
 
-        subprocess.check_call(command, cwd=SRC_ROOT / "glean-core" / "ffi", env=env)
+        subprocess.check_call(command, cwd=SRC_ROOT / "glean-core", env=env)
         shutil.copyfile(
             SRC_ROOT / "target" / target / buildvariant / shared_object,
             PYTHON_ROOT / "glean" / shared_object,
         )
 
+        command = [
+            "cargo",
+            "uniffi-bindgen",
+            "generate",
+            "glean-core/src/glean.udl",
+            "--language",
+            "python",
+            "--out-dir",
+            SRC_ROOT / "target",
+        ]
+        subprocess.check_call(command, cwd=SRC_ROOT, env=env)
+        shutil.copyfile(
+            SRC_ROOT / "target" / "glean.py", PYTHON_ROOT / "glean" / "_uniffi.py"
+        )
+
         shutil.copyfile(
             PYTHON_ROOT.parent / "metrics.yaml", PYTHON_ROOT / "glean" / "metrics.yaml"
         )
@@ -245,8 +255,7 @@ setup(
         "glean.net": FROM_TOP / "glean" / "net",
         "glean.testing": FROM_TOP / "glean" / "testing",
     },
-    setup_requires=setup_requirements,
-    cffi_modules=[str(PYTHON_ROOT / "ffi_build.py:ffibuilder")],
+    setup_requires=[],
     url="https://github.com/mozilla/glean",
     zip_safe=False,
     package_data={"glean": [shared_object, "metrics.yaml", "pings.yaml"]},
