@@ -5,17 +5,16 @@
 import enum
 from pathlib import Path
 import time
-from typing import List, Optional, Tuple
+from typing import Dict, Optional
 
 
 import pytest
 
 
 from glean import Configuration, Glean
-from glean._dispatcher import Dispatcher
 from glean import load_metrics
 from glean import metrics
-from glean.metrics import Lifetime
+from glean.metrics import Lifetime, CommonMetricData
 from glean import testing
 from glean import __version__ as glean_version
 
@@ -23,37 +22,53 @@ from glean import __version__ as glean_version
 ROOT = Path(__file__).parent
 
 
+class ClickKeys(metrics.EventExtras):
+    def __init__(
+        self, object_id: Optional[str] = None, other: Optional[str] = None
+    ) -> None:
+        self._object_id = object_id
+        self._other = other
+
+    def to_ffi_extra(self) -> Dict[str, str]:
+        extras = {}
+        if self._object_id is not None:
+            extras["object_id"] = str(self._object_id)
+
+        if self._other is not None:
+            extras["other"] = str(self._other)
+
+        return extras
+
+
 def test_the_api_records_to_its_storage_engine():
-    class ClickKeys(enum.Enum):
-        OBJECT_ID = 0
-        OTHER = 1
 
     click = metrics.EventMetricType(
-        disabled=False,
-        category="ui",
-        lifetime=Lifetime.PING,
-        name="click",
-        send_in_pings=["store1"],
+        CommonMetricData(
+            disabled=False,
+            category="ui",
+            lifetime=Lifetime.PING,
+            name="click",
+            send_in_pings=["store1"],
+            dynamic_label=None,
+        ),
         allowed_extra_keys=["object_id", "other"],
     )
 
     # Record two events of the same type, with a little delay
-    click.record({ClickKeys.OBJECT_ID: "buttonA", ClickKeys.OTHER: "foo"})
+    click.record(ClickKeys(object_id="buttonA", other="foo"))
 
     time.sleep(0.001)
 
-    click.record({ClickKeys.OBJECT_ID: "buttonB", ClickKeys.OTHER: "bar"})
+    click.record(ClickKeys(object_id="buttonB", other="bar"))
     click.record()
 
     # Check that data was properly recorded
     snapshot = click.test_get_value()
-    assert click.test_has_value()
     assert 3 == len(snapshot)
 
     first_event = [x for x in snapshot if x.extra.get("object_id") == "buttonA"][0]
     assert "ui" == first_event.category
     assert "click" == first_event.name
-    assert "ui.click" == first_event.identifier
     assert "foo" == first_event.extra["other"]
 
     second_event = [x for x in snapshot if x.extra.get("object_id") == "buttonB"][0]
@@ -65,28 +80,28 @@ def test_the_api_records_to_its_storage_engine():
 
 
 def test_the_api_records_to_its_storage_engine_when_category_is_empty():
-    class ClickKeys(enum.Enum):
-        OBJECT_ID = 0
 
     click = metrics.EventMetricType(
-        disabled=False,
-        category="",
-        lifetime=Lifetime.PING,
-        name="click",
-        send_in_pings=["store1"],
+        CommonMetricData(
+            disabled=False,
+            category="",
+            lifetime=Lifetime.PING,
+            name="click",
+            send_in_pings=["store1"],
+            dynamic_label=None,
+        ),
         allowed_extra_keys=["object_id"],
     )
 
     # Record two events of the same type, with a little delay
-    click.record(extra={ClickKeys.OBJECT_ID: "buttonA"})
+    click.record(ClickKeys(object_id="buttonA"))
 
     time.sleep(0.001)
 
-    click.record(extra={ClickKeys.OBJECT_ID: "buttonB"})
+    click.record(ClickKeys(object_id="buttonB"))
 
     # Check that the data was properly recorded
     snapshot = click.test_get_value()
-    assert click.test_has_value()
     assert 2 == len(snapshot)
 
     first_event = [x for x in snapshot if x.extra["object_id"] == "buttonA"][0]
@@ -99,16 +114,16 @@ def test_the_api_records_to_its_storage_engine_when_category_is_empty():
 
 
 def test_disabled_events_must_not_record_data():
-    class ClickKeys(enum.Enum):
-        OBJECT_ID = 0
-        OTHER = 1
 
     click = metrics.EventMetricType(
-        disabled=True,
-        category="ui",
-        lifetime=Lifetime.PING,
-        name="click",
-        send_in_pings=["store1"],
+        CommonMetricData(
+            disabled=True,
+            category="ui",
+            lifetime=Lifetime.PING,
+            name="click",
+            send_in_pings=["store1"],
+            dynamic_label=None,
+        ),
         allowed_extra_keys=["object_id", "other"],
     )
 
@@ -116,46 +131,47 @@ def test_disabled_events_must_not_record_data():
     click.record()
 
     # Check that nothing was recorded
-    assert not click.test_has_value()
+    assert not click.test_get_value()
 
 
 def test_test_get_value_throws_valueerror_if_nothing_is_stored():
     click = metrics.EventMetricType(
-        disabled=False,
-        category="ui",
-        lifetime=Lifetime.PING,
-        name="click",
-        send_in_pings=["store1"],
+        CommonMetricData(
+            disabled=False,
+            category="ui",
+            lifetime=Lifetime.PING,
+            name="click",
+            send_in_pings=["store1"],
+            dynamic_label=None,
+        ),
         allowed_extra_keys=["object_id", "other"],
     )
 
-    with pytest.raises(ValueError):
-        click.test_get_value()
+    assert not click.test_get_value()
 
 
 def test_the_api_records_to_secondary_pings():
-    class ClickKeys(enum.Enum):
-        OBJECT_ID = 0
-
     click = metrics.EventMetricType(
-        disabled=False,
-        category="ui",
-        lifetime=Lifetime.PING,
-        name="click",
-        send_in_pings=["store1", "store2"],
+        CommonMetricData(
+            disabled=False,
+            category="ui",
+            lifetime=Lifetime.PING,
+            name="click",
+            send_in_pings=["store1", "store2"],
+            dynamic_label=None,
+        ),
         allowed_extra_keys=["object_id"],
     )
 
     # Record two events of the same type, with a little delay
-    click.record(extra={ClickKeys.OBJECT_ID: "buttonA"})
+    click.record(ClickKeys(object_id="buttonA"))
 
     time.sleep(0.001)
 
-    click.record(extra={ClickKeys.OBJECT_ID: "buttonB"})
+    click.record(ClickKeys(object_id="buttonB"))
 
     # Check that the data was properly recorded in the second ping
     snapshot = click.test_get_value("store2")
-    assert click.test_has_value("store2")
     assert 2 == len(snapshot)
 
     first_event = [x for x in snapshot if x.extra["object_id"] == "buttonA"][0]
@@ -171,14 +187,17 @@ def test_the_api_records_to_secondary_pings():
 
 def test_events_should_not_record_when_upload_is_disabled():
     class EventKeys(enum.Enum):
-        TEST_NAME = 0
+        TEST_NAME = "test_name"
 
     event_metric = metrics.EventMetricType(
-        disabled=False,
-        category="ui",
-        lifetime=Lifetime.PING,
-        name="click",
-        send_in_pings=["store1"],
+        CommonMetricData(
+            disabled=False,
+            category="ui",
+            lifetime=Lifetime.PING,
+            name="click",
+            send_in_pings=["store1"],
+            dynamic_label=None,
+        ),
         allowed_extra_keys=["test_name"],
     )
 
@@ -186,10 +205,11 @@ def test_events_should_not_record_when_upload_is_disabled():
     event_metric.record({EventKeys.TEST_NAME: "event1"})
     snapshot1 = event_metric.test_get_value()
     assert 1 == len(snapshot1)
+
     Glean.set_upload_enabled(False)
     event_metric.record({EventKeys.TEST_NAME: "event2"})
-    with pytest.raises(ValueError):
-        event_metric.test_get_value()
+    assert not event_metric.test_get_value()
+
     Glean.set_upload_enabled(True)
     event_metric.record({EventKeys.TEST_NAME: "event3"})
     snapshot3 = event_metric.test_get_value()
@@ -202,14 +222,17 @@ def test_flush_queued_events_on_startup(safe_httpserver):
     Glean._configuration.server_endpoint = safe_httpserver.url
 
     class EventKeys(enum.Enum):
-        SOME_EXTRA = 0
+        SOME_EXTRA = "some_extra"
 
     event = metrics.EventMetricType(
-        disabled=False,
-        category="telemetry",
-        lifetime=Lifetime.PING,
-        name="test_event",
-        send_in_pings=["events"],
+        CommonMetricData(
+            disabled=False,
+            category="telemetry",
+            lifetime=Lifetime.PING,
+            name="test_event",
+            send_in_pings=["events"],
+            dynamic_label=None,
+        ),
         allowed_extra_keys=["some_extra"],
     )
 
@@ -229,6 +252,8 @@ def test_flush_queued_events_on_startup(safe_httpserver):
     assert "events" in request.url
 
 
+# Dispatcher usage
+@pytest.mark.skip
 def test_flush_queued_events_on_startup_and_correctly_handle_preinit_events(
     safe_httpserver,
 ):
@@ -237,21 +262,24 @@ def test_flush_queued_events_on_startup_and_correctly_handle_preinit_events(
     Glean._configuration.server_endpoint = safe_httpserver.url
 
     class EventKeys(enum.Enum):
-        SOME_EXTRA = 0
+        SOME_EXTRA = "some_extra"
 
     event = metrics.EventMetricType(
-        disabled=False,
-        category="telemetry",
-        lifetime=Lifetime.PING,
-        name="test_event",
-        send_in_pings=["events"],
+        CommonMetricData(
+            disabled=False,
+            category="telemetry",
+            lifetime=Lifetime.PING,
+            name="test_event",
+            send_in_pings=["events"],
+            dynamic_label=None,
+        ),
         allowed_extra_keys=["some_extra"],
     )
 
     event.record(extra={EventKeys.SOME_EXTRA: "run1"})
     assert 1 == len(event.test_get_value())
 
-    Dispatcher.set_task_queueing(True)
+    # Dispatcher.set_task_queueing(True)
     event.record(extra={EventKeys.SOME_EXTRA: "pre-init"})
 
     testing.reset_glean(
@@ -277,22 +305,22 @@ def test_flush_queued_events_on_startup_and_correctly_handle_preinit_events(
 
 
 def test_long_extra_values_record_an_error():
-    class ClickKeys(enum.Enum):
-        OBJECT_ID = 0
-        OTHER = 1
 
     click = metrics.EventMetricType(
-        disabled=False,
-        category="ui",
-        lifetime=Lifetime.PING,
-        name="click",
-        send_in_pings=["store1"],
+        CommonMetricData(
+            disabled=False,
+            category="ui",
+            lifetime=Lifetime.PING,
+            name="click",
+            send_in_pings=["store1"],
+            dynamic_label=None,
+        ),
         allowed_extra_keys=["object_id", "other"],
     )
 
-    long_string = "0123456789" * 11
+    long_string = "0123456789" * 51
 
-    click.record(extra={ClickKeys.OBJECT_ID: long_string})
+    click.record(ClickKeys(object_id=long_string))
 
     assert 1 == click.test_get_num_recorded_errors(testing.ErrorType.INVALID_OVERFLOW)
 
@@ -304,10 +332,7 @@ def test_event_enum_is_generated_correctly():
 
     print(dir(metrics.environment))
     metrics.environment.event_example.record(
-        {
-            metrics.environment.event_example_keys.KEY1: "value1",
-            metrics.environment.event_example_keys.KEY2: "value2",
-        }
+        metrics.environment.EventExampleExtra(key1="value1", key2="value2")
     )
 
     assert {
@@ -318,7 +343,8 @@ def test_event_enum_is_generated_correctly():
 
 def test_event_extra_is_generated_correctly():
     metrics = load_metrics(
-        ROOT.parent / "data" / "events_with_types.yaml", config={"allow_reserved": False}
+        ROOT.parent / "data" / "events_with_types.yaml",
+        config={"allow_reserved": False},
     )
 
     metrics.core.preference_toggled.record(
@@ -332,31 +358,16 @@ def test_event_extra_is_generated_correctly():
 
 
 def test_the_convenient_extrakeys_api():
-    class ClickKeys(metrics.EventExtras):
-        def __init__(self, object_id: Optional[str] = None, other: Optional[str] = None) -> None:
-            self._object_id = object_id
-            self._other = other
-
-        def to_ffi_extra(self) -> Tuple[List[int], List[str]]:
-            keys = []
-            values = []
-
-            if self._object_id is not None:
-                keys.append(0)
-                values.append(self._object_id)
-
-            if self._other is not None:
-                keys.append(1)
-                values.append(self._other)
-
-            return (keys, values)
 
     click = metrics.EventMetricType(
-        disabled=False,
-        category="ui",
-        lifetime=Lifetime.PING,
-        name="click",
-        send_in_pings=["store1"],
+        CommonMetricData(
+            disabled=False,
+            category="ui",
+            lifetime=Lifetime.PING,
+            name="click",
+            send_in_pings=["store1"],
+            dynamic_label=None,
+        ),
         allowed_extra_keys=["object_id", "other"],
     )
 
@@ -371,13 +382,11 @@ def test_the_convenient_extrakeys_api():
 
     # Check that data was properly recorded
     snapshot = click.test_get_value()
-    assert click.test_has_value()
     assert 3 == len(snapshot)
 
     first_event = [x for x in snapshot if x.extra.get("object_id") == "buttonA"][0]
     assert "ui" == first_event.category
     assert "click" == first_event.name
-    assert "ui.click" == first_event.identifier
     assert "foo" == first_event.extra["other"]
 
     second_event = [x for x in snapshot if x.extra.get("object_id") == "buttonB"][0]
@@ -390,7 +399,8 @@ def test_the_convenient_extrakeys_api():
 
 def test_event_extra_does_typechecks():
     metrics = load_metrics(
-        ROOT.parent / "data" / "events_with_types.yaml", config={"allow_reserved": False}
+        ROOT.parent / "data" / "events_with_types.yaml",
+        config={"allow_reserved": False},
     )
 
     # Valid combinations of extras.
@@ -398,10 +408,16 @@ def test_event_extra_does_typechecks():
     metrics.core.PreferenceToggledExtra(preference="value1")
     metrics.core.PreferenceToggledExtra(enabled=True)
     metrics.core.PreferenceToggledExtra(swapped=1)
-    extras = metrics.core.PreferenceToggledExtra(preference="value1", enabled=True, swapped=1)
+    extras = metrics.core.PreferenceToggledExtra(
+        preference="value1", enabled=True, swapped=1
+    )
     # Check conversion to FFI types, extras are sorted by name
     ffi = extras.to_ffi_extra()
-    expected = ([0, 1, 2], ["true", "value1", "1"])
+    expected = {
+        "preference": "value1",
+        "enabled": "true",
+        "swapped": "1",
+    }
     assert expected == ffi
 
     with pytest.raises(TypeError):
