@@ -16,9 +16,6 @@ import uuid
 
 
 from glean_parser import validate_ping
-from glean_parser.metrics import Lifetime as ParserLifetime
-from glean_parser.metrics import TimeUnit as ParserTimeUnit
-from glean_parser.metrics import MemoryUnit as ParserMemoryUnit
 import pytest
 
 
@@ -26,17 +23,16 @@ from glean import Configuration, Glean, load_metrics
 from glean import __version__ as glean_version
 from glean import _builtins
 from glean import _util
-from glean._dispatcher import Dispatcher
 from glean.metrics import (
     CounterMetricType,
+    CommonMetricData,
     Lifetime,
-    MemoryUnit,
     PingType,
     StringMetricType,
-    TimeUnit,
 )
 from glean.net import PingUploadWorker
 from glean.testing import _RecordingUploader
+from glean._uniffi import glean_set_test_mode
 
 GLEAN_APP_ID = "glean-python-test"
 
@@ -60,11 +56,14 @@ def test_submit_a_ping(safe_httpserver):
     Glean._configuration.server_endpoint = safe_httpserver.url
 
     counter_metric = CounterMetricType(
-        disabled=False,
-        category="telemetry",
-        lifetime=Lifetime.APPLICATION,
-        name="counter_metric",
-        send_in_pings=["baseline"],
+        CommonMetricData(
+            disabled=False,
+            category="telemetry",
+            lifetime=Lifetime.APPLICATION,
+            name="counter_metric",
+            send_in_pings=["baseline"],
+            dynamic_label=None,
+        )
     )
 
     counter_metric.add()
@@ -80,22 +79,25 @@ def test_submit_a_ping(safe_httpserver):
 def test_submiting_an_empty_ping_doesnt_queue_work(safe_httpserver):
     safe_httpserver.serve_content(b"", code=200)
 
-    Glean._submit_ping_by_name("metrics")
+    _builtins.pings.metrics.submit()
     assert 0 == len(safe_httpserver.requests)
 
 
 def test_disabling_upload_should_disable_metrics_recording():
     counter_metric = CounterMetricType(
-        disabled=False,
-        category="telemetry",
-        lifetime=Lifetime.APPLICATION,
-        name="counter_metric",
-        send_in_pings=["store1"],
+        CommonMetricData(
+            disabled=False,
+            category="telemetry",
+            lifetime=Lifetime.APPLICATION,
+            name="counter_metric",
+            send_in_pings=["store1"],
+            dynamic_label=None,
+        )
     )
 
     Glean.set_upload_enabled(False)
     counter_metric.add(1)
-    assert False is counter_metric.test_has_value()
+    assert None is counter_metric.test_get_value()
 
 
 def test_experiments_recording():
@@ -160,24 +162,24 @@ def test_initialize_must_not_crash_if_data_dir_is_messed_up(tmpdir):
         data_dir=filename,
     )
 
-    # This should cause initialization to fail
-    assert False is Glean.is_initialized()
+    # This should cause initialization to fail,
+    # but we don't have a way to check.
 
-    shutil.rmtree(str(tmpdir))
+    shutil.rmtree(str(tmpdir), ignore_errors=True)
 
 
 def test_queued_recorded_metrics_correctly_during_init():
     Glean._reset()
 
-    # Enable queueing
-    Dispatcher.set_task_queueing(True)
-
     counter_metric = CounterMetricType(
-        disabled=False,
-        category="telemetry",
-        lifetime=Lifetime.APPLICATION,
-        name="counter_metric",
-        send_in_pings=["store1"],
+        CommonMetricData(
+            disabled=False,
+            category="telemetry",
+            lifetime=Lifetime.APPLICATION,
+            name="counter_metric",
+            send_in_pings=["store1"],
+            dynamic_label=None,
+        )
     )
 
     for _ in range(2):
@@ -189,7 +191,6 @@ def test_queued_recorded_metrics_correctly_during_init():
         upload_enabled=True,
     )
 
-    assert counter_metric.test_has_value()
     assert 2 == counter_metric.test_get_value()
 
 
@@ -214,11 +215,14 @@ def test_dont_schedule_pings_if_metrics_disabled(safe_httpserver):
     safe_httpserver.serve_content(b"", code=200)
 
     counter_metric = CounterMetricType(
-        disabled=False,
-        category="telemetry",
-        lifetime=Lifetime.APPLICATION,
-        name="counter_metric",
-        send_in_pings=["store1"],
+        CommonMetricData(
+            disabled=False,
+            category="telemetry",
+            lifetime=Lifetime.APPLICATION,
+            name="counter_metric",
+            send_in_pings=["store1"],
+            dynamic_label=None,
+        )
     )
 
     custom_ping = PingType(
@@ -303,16 +307,20 @@ def test_ping_collection_must_happen_after_currently_scheduled_metrics_recording
         name=ping_name, include_client_id=True, send_if_empty=False, reason_codes=[]
     )
     string_metric = StringMetricType(
-        disabled=False,
-        category="category",
-        lifetime=Lifetime.PING,
-        name="string_metric",
-        send_in_pings=[ping_name],
+        CommonMetricData(
+            disabled=False,
+            category="category",
+            lifetime=Lifetime.PING,
+            name="string_metric",
+            send_in_pings=[ping_name],
+            dynamic_label=None,
+        )
     )
 
     # This test relies on testing mode to be disabled, since we need to prove the
     # real-world async behaviour of this.
-    Dispatcher._testing_mode = False
+    Glean._testing_mode = False
+    glean_set_test_mode(False)
 
     # This is the important part of the test. Even though both the metrics API and
     # sendPings are async and off the main thread, "SomeTestValue" should be recorded,
@@ -321,9 +329,6 @@ def test_ping_collection_must_happen_after_currently_scheduled_metrics_recording
     string_metric.set(test_value)
     ping.submit()
 
-    # Wait until the work is complete
-    Dispatcher._task_worker._queue.join()
-
     while not info_path.exists():
         time.sleep(0.1)
 
@@ -346,61 +351,36 @@ def test_ping_collection_must_happen_after_currently_scheduled_metrics_recording
 
 def test_basic_metrics_should_be_cleared_when_disabling_uploading():
     counter_metric = CounterMetricType(
-        disabled=False,
-        category="telemetry",
-        lifetime=Lifetime.APPLICATION,
-        name="counter_metric",
-        send_in_pings=["store1"],
+        CommonMetricData(
+            disabled=False,
+            category="telemetry",
+            lifetime=Lifetime.APPLICATION,
+            name="counter_metric",
+            send_in_pings=["store1"],
+            dynamic_label=None,
+        )
     )
 
     counter_metric.add(10)
-    assert counter_metric.test_has_value()
+    assert 10 == counter_metric.test_get_value()
 
     Glean.set_upload_enabled(False)
-    assert not counter_metric.test_has_value()
+    assert not counter_metric.test_get_value()
     counter_metric.add(10)
-    assert not counter_metric.test_has_value()
+    assert not counter_metric.test_get_value()
 
     Glean.set_upload_enabled(True)
-    assert not counter_metric.test_has_value()
+    assert not counter_metric.test_get_value()
     counter_metric.add(10)
-    assert counter_metric.test_has_value()
+    assert 10 == counter_metric.test_get_value()
 
 
 def test_core_metrics_should_be_cleared_with_disabling_and_enabling_uploading():
-    assert _builtins.metrics.glean.internal.metrics.os.test_has_value()
+    assert _builtins.metrics.glean.internal.metrics.os.test_get_value()
     Glean.set_upload_enabled(False)
-    assert not _builtins.metrics.glean.internal.metrics.os.test_has_value()
+    assert not _builtins.metrics.glean.internal.metrics.os.test_get_value()
     Glean.set_upload_enabled(True)
-    assert _builtins.metrics.glean.internal.metrics.os.test_has_value()
-
-
-def test_collect(ping_schema_url):
-    counter_metric = CounterMetricType(
-        disabled=False,
-        category="telemetry",
-        lifetime=Lifetime.APPLICATION,
-        name="counter_metric",
-        send_in_pings=["store1"],
-    )
-
-    custom_ping = PingType(
-        name="store1", include_client_id=True, send_if_empty=False, reason_codes=[]
-    )
-
-    counter_metric.add(10)
-
-    json_content = Glean.test_collect(custom_ping)
-
-    assert isinstance(json_content, str)
-
-    json_tree = json.loads(json_content)
-
-    assert 10 == json_tree["metrics"]["counter"]["telemetry.counter_metric"]
-
-    assert 0 == validate_ping.validate_ping(
-        io.StringIO(json_content), sys.stdout, schema_url=ping_schema_url
-    )
+    assert _builtins.metrics.glean.internal.metrics.os.test_get_value()
 
 
 def test_tempdir_is_cleared():
@@ -453,6 +433,15 @@ def test_set_application_build_id():
     )
 
 
+def wait_for_requests(server, n=1, timeout=2):
+    start_time = time.time()
+    while len(server.requests) < n:
+        time.sleep(0.1)
+        if time.time() - start_time > timeout:
+            raise TimeoutError()
+
+
+@pytest.mark.skipif(sys.platform == "win32", reason="bug 1771157: Windows failures")
 def test_set_application_id_and_version(safe_httpserver):
     safe_httpserver.serve_content(b"", code=200)
     Glean._reset()
@@ -469,9 +458,8 @@ def test_set_application_id_and_version(safe_httpserver):
         == _builtins.metrics.glean.internal.metrics.app_display_version.test_get_value()
     )
 
-    Glean._configuration.server_endpoint = safe_httpserver.url
-
     _builtins.pings.baseline.submit()
+    wait_for_requests(safe_httpserver)
 
     assert 1 == len(safe_httpserver.requests)
 
@@ -492,36 +480,20 @@ def test_disabling_upload_sends_deletion_request(safe_httpserver):
     assert 1 == len(safe_httpserver.requests)
 
 
-def test_overflowing_the_task_queue_records_telemetry():
-    Dispatcher.set_task_queueing(True)
-
-    for _ in range(110):
-        Dispatcher.launch(lambda: None)
-
-    assert 100 == len(Dispatcher._preinit_task_queue)
-    assert 10 == Dispatcher._overflow_count
-
-    Dispatcher.flush_queued_initial_tasks()
-
-    assert 110 == _builtins.metrics.glean.error.preinit_tasks_overflow.test_get_value()
-
-    json_content = Glean.test_collect(_builtins.pings.metrics)
-    json_tree = json.loads(json_content)
-
-    assert 110 == json_tree["metrics"]["counter"]["glean.error.preinit_tasks_overflow"]
-
-
 def test_configuration_property(safe_httpserver):
     safe_httpserver.serve_content(b"", code=200)
 
     Glean._configuration.server_endpoint = safe_httpserver.url
 
     counter_metric = CounterMetricType(
-        disabled=False,
-        category="telemetry",
-        lifetime=Lifetime.APPLICATION,
-        name="counter_metric",
-        send_in_pings=["baseline"],
+        CommonMetricData(
+            disabled=False,
+            category="telemetry",
+            lifetime=Lifetime.APPLICATION,
+            name="counter_metric",
+            send_in_pings=["baseline"],
+            dynamic_label=None,
+        )
     )
 
     counter_metric.add()
@@ -642,11 +614,14 @@ def test_clear_application_lifetime_metrics(tmpdir):
     )
 
     counter_metric = CounterMetricType(
-        disabled=False,
-        category="test.telemetry",
-        lifetime=Lifetime.APPLICATION,
-        name="lifetime_reset",
-        send_in_pings=["store1"],
+        CommonMetricData(
+            disabled=False,
+            category="test.telemetry",
+            lifetime=Lifetime.APPLICATION,
+            name="lifetime_reset",
+            send_in_pings=["store1"],
+            dynamic_label=None,
+        )
     )
 
     # Additionally get metrics using the loader.
@@ -655,10 +630,8 @@ def test_clear_application_lifetime_metrics(tmpdir):
     counter_metric.add(10)
     metrics.core_ping.seq.add(10)
 
-    assert counter_metric.test_has_value()
     assert counter_metric.test_get_value() == 10
 
-    assert metrics.core_ping.seq.test_has_value()
     assert metrics.core_ping.seq.test_get_value() == 10
 
     Glean._reset()
@@ -670,25 +643,8 @@ def test_clear_application_lifetime_metrics(tmpdir):
         data_dir=Path(str(tmpdir)),
     )
 
-    assert not counter_metric.test_has_value()
-    assert not metrics.core_ping.seq.test_has_value()
-
-
-def test_confirm_enums_match_values_in_glean_parser():
-    """
-    Make sure the values in the glean_parser enums match those in Glean's enums
-    (which come directly from the canonical source in the Rust implementation).
-
-    This should ensure we never update to a glean_parser version with incorrect
-    enumeration values.
-    """
-    for g_enum, gp_enum in [
-        (Lifetime, ParserLifetime),
-        (TimeUnit, ParserTimeUnit),
-        (MemoryUnit, ParserMemoryUnit),
-    ]:
-        for name in gp_enum.__members__.keys():
-            assert g_enum[name.upper()].value == gp_enum[name].value
+    assert not counter_metric.test_get_value()
+    assert not metrics.core_ping.seq.test_get_value()
 
 
 def test_presubmit_makes_a_valid_ping(tmpdir, ping_schema_url, monkeypatch):
@@ -697,6 +653,8 @@ def test_presubmit_makes_a_valid_ping(tmpdir, ping_schema_url, monkeypatch):
 
     info_path = Path(str(tmpdir)) / "info.txt"
 
+    # This test relies on testing mode to be disabled, since we need to prove the
+    # real-world async behaviour of this.
     Glean._reset()
 
     ping_name = "preinit_ping"
@@ -704,11 +662,6 @@ def test_presubmit_makes_a_valid_ping(tmpdir, ping_schema_url, monkeypatch):
         name=ping_name, include_client_id=True, send_if_empty=True, reason_codes=[]
     )
 
-    # This test relies on testing mode to be disabled, since we need to prove the
-    # real-world async behaviour of this.
-    Dispatcher._testing_mode = False
-    Dispatcher._queue_initial_tasks = True
-
     # Submit a ping prior to calling initialize
     ping.submit()
     Glean._initialize_with_tempdir_for_testing(
@@ -722,9 +675,6 @@ def test_presubmit_makes_a_valid_ping(tmpdir, ping_schema_url, monkeypatch):
         Glean._configuration, "ping_uploader", _RecordingUploader(info_path)
     )
 
-    # Wait until the work is complete
-    Dispatcher._task_worker._queue.join()
-
     while not info_path.exists():
         time.sleep(0.1)
 
@@ -732,7 +682,6 @@ def test_presubmit_makes_a_valid_ping(tmpdir, ping_schema_url, monkeypatch):
         url_path = fd.readline()
         serialized_ping = fd.readline()
 
-    print(url_path)
     assert ping_name == url_path.split("/")[3]
 
     assert 0 == validate_ping.validate_ping(
@@ -758,7 +707,10 @@ def test_app_display_version_unknown():
     )
 
 
+@pytest.mark.skipif(sys.platform == "win32", reason="bug 1771157: Windows failures")
 def test_flipping_upload_enabled_respects_order_of_events(tmpdir, monkeypatch):
+    # This test relies on testing mode to be disabled, since we need to prove the
+    # real-world async behaviour of this.
     Glean._reset()
 
     info_path = Path(str(tmpdir)) / "info.txt"
@@ -771,11 +723,6 @@ def test_flipping_upload_enabled_respects_order_of_events(tmpdir, monkeypatch):
         reason_codes=[],
     )
 
-    # This test relies on testing mode to be disabled, since we need to prove the
-    # real-world async behaviour of this.
-    Dispatcher._testing_mode = False
-    Dispatcher._queue_initial_tasks = True
-
     configuration = Glean._configuration
     configuration.ping_uploader = _RecordingUploader(info_path)
     Glean._initialize_with_tempdir_for_testing(
@@ -790,14 +737,7 @@ def test_flipping_upload_enabled_respects_order_of_events(tmpdir, monkeypatch):
     # Submit a custom ping.
     ping.submit()
 
-    # Wait until the work is complete
-    Dispatcher._task_worker._queue.join()
-
-    while not info_path.exists():
-        time.sleep(0.1)
-
-    with info_path.open("r") as fd:
-        url_path = fd.readline()
+    url_path, payload = wait_for_ping(info_path, max_wait=20)
 
     # Validate we got the deletion-request ping
     assert "deletion-request" == url_path.split("/")[3]
@@ -834,6 +774,7 @@ def wait_for_ping(info_path, max_wait=10) -> (str, str):
     return (url_path, payload)
 
 
+@pytest.mark.skipif(sys.platform == "win32", reason="bug 1771157: Windows failures")
 def test_client_activity_api(tmpdir, monkeypatch):
     Glean._reset()
 
@@ -844,6 +785,9 @@ def test_client_activity_api(tmpdir, monkeypatch):
 
     configuration = Glean._configuration
     configuration.ping_uploader = _RecordingUploader(info_path)
+
+    Glean._testing_mode = False
+    glean_set_test_mode(False)
     Glean._initialize_with_tempdir_for_testing(
         application_id=GLEAN_APP_ID,
         application_version=glean_version,
@@ -851,29 +795,33 @@ def test_client_activity_api(tmpdir, monkeypatch):
         configuration=Glean._configuration,
     )
 
-    # Wait until the work is complete
-    Dispatcher._task_worker._queue.join()
-
     # Making it active
     Glean.handle_client_active()
 
-    url_path, payload = wait_for_ping(info_path)
+    url_path, payload = wait_for_ping(info_path, max_wait=20)
     assert "baseline" == url_path.split("/")[3]
     assert payload["ping_info"]["reason"] == "active"
-    assert "timespan" not in payload["metrics"]
+    # It's an empty ping.
+    assert "metrics" not in payload
 
+    # The upload process is fast, but not fast enough to communicate its status.
+    # We give it just a blink of an eye to wind down.
+    time.sleep(0.1)
     # Making it inactive
     Glean.handle_client_inactive()
 
-    url_path, payload = wait_for_ping(info_path)
+    url_path, payload = wait_for_ping(info_path, max_wait=20)
     assert "baseline" == url_path.split("/")[3]
     assert payload["ping_info"]["reason"] == "inactive"
     assert "glean.baseline.duration" in payload["metrics"]["timespan"]
 
+    # The upload process is fast, but not fast enough to communicate its status.
+    # We give it just a blink of an eye to wind down.
+    time.sleep(0.1)
     # Once more active
     Glean.handle_client_active()
 
-    url_path, payload = wait_for_ping(info_path)
+    url_path, payload = wait_for_ping(info_path, max_wait=20)
     assert "baseline" == url_path.split("/")[3]
     assert payload["ping_info"]["reason"] == "active"
     assert "timespan" not in payload["metrics"]
@@ -884,11 +832,14 @@ def test_sending_of_custom_pings(safe_httpserver):
     Glean._configuration.server_endpoint = safe_httpserver.url
 
     counter_metric = CounterMetricType(
-        disabled=False,
-        category="telemetry",
-        lifetime=Lifetime.APPLICATION,
-        name="counter_metric",
-        send_in_pings=["store1"],
+        CommonMetricData(
+            disabled=False,
+            category="telemetry",
+            lifetime=Lifetime.APPLICATION,
+            name="counter_metric",
+            send_in_pings=["store1"],
+            dynamic_label=None,
+        )
     )
 
     custom_ping = PingType(
