@@ -2,9 +2,11 @@
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at https://mozilla.org/MPL/2.0/.
 
+use std::sync::{Arc, Barrier, Mutex};
+use std::thread::{self, ThreadId};
+
 use crate::private::PingType;
-use crate::private::{BooleanMetric, CounterMetric, EventMetric};
-use std::path::PathBuf;
+use crate::private::{BooleanMetric, CounterMetric, EventMetric, StringMetric};
 
 use super::*;
 use crate::common_test::{lock_test, new_glean, GLOBAL_APPLICATION_ID};
@@ -29,13 +31,13 @@ fn send_a_ping() {
             _headers: Vec<(String, String)>,
         ) -> net::UploadResult {
             self.sender.send(url).unwrap();
-            net::UploadResult::HttpStatus(200)
+            net::UploadResult::http_status(200)
         }
     }
 
     // Create a custom configuration to use a fake uploader.
     let dir = tempfile::tempdir().unwrap();
-    let tmpname = dir.path().display().to_string();
+    let tmpname = dir.path().to_path_buf();
 
     let cfg = Configuration {
         data_path: tmpname,
@@ -43,13 +45,13 @@ fn send_a_ping() {
         upload_enabled: true,
         max_events: None,
         delay_ping_lifetime_io: false,
-        channel: Some("testing".into()),
         server_endpoint: Some("invalid-test-host".into()),
         uploader: Some(Box::new(FakeUploader { sender: s })),
+        use_core_mps: false,
+        trim_data_to_registered_pings: false,
     };
 
     let _t = new_glean(Some(cfg), true);
-    crate::block_on_dispatcher();
 
     // Define a new ping and submit it.
     const PING_NAME: &str = "test-ping";
@@ -58,7 +60,7 @@ fn send_a_ping() {
 
     // Wait for the ping to arrive.
     let url = r.recv().unwrap();
-    assert_eq!(url.contains(PING_NAME), true);
+    assert!(url.contains(PING_NAME));
 }
 
 #[test]
@@ -66,7 +68,6 @@ fn disabling_upload_disables_metrics_recording() {
     let _lock = lock_test();
 
     let _t = new_glean(None, true);
-    crate::block_on_dispatcher();
 
     let metric = BooleanMetric::new(CommonMetricData {
         name: "bool_metric".into(),
@@ -79,7 +80,7 @@ fn disabling_upload_disables_metrics_recording() {
 
     crate::set_upload_enabled(false);
 
-    assert!(metric.test_get_value("store1").is_none())
+    assert!(metric.test_get_value(Some("store1".into())).is_none())
 }
 
 #[test]
@@ -101,7 +102,7 @@ fn test_experiments_recording() {
     set_experiment_inactive("experiment_test".to_string());
     assert!(!test_is_experiment_active("experiment_test".to_string()));
     assert!(test_is_experiment_active("experiment_api".to_string()));
-    let stored_data = test_get_experiment_data("experiment_api".to_string());
+    let stored_data = test_get_experiment_data("experiment_api".to_string()).unwrap();
     assert_eq!("branch_b", stored_data.branch);
     assert_eq!("value", stored_data.extra.unwrap()["test_key"]);
 }
@@ -109,27 +110,10 @@ fn test_experiments_recording() {
 #[test]
 fn test_experiments_recording_before_glean_inits() {
     let _lock = lock_test();
+    let dir = tempfile::tempdir().unwrap();
+    let tmpname = dir.path().to_path_buf();
 
-    // Destroy the existing glean instance from glean-core so that we
-    // can test the pre-init queueing of the experiment api commands.
-    // This is doing the exact same thing that `reset_glean` is doing
-    // but without calling `initialize`.
-    if was_initialize_called() {
-        // We need to check if the Glean object (from glean-core) is
-        // initialized, otherwise this will crash on the first test
-        // due to bug 1675215 (this check can be removed once that
-        // bug is fixed).
-        if global_glean().is_some() {
-            with_glean_mut(|glean| {
-                glean.test_clear_all_stores();
-                glean.destroy_db();
-            });
-        }
-        // Allow us to go through initialization again.
-        INITIALIZE_CALLED.store(false, Ordering::SeqCst);
-        // Reset the dispatcher.
-        dispatcher::reset_dispatcher();
-    }
+    destroy_glean(true, &tmpname);
 
     set_experiment_active(
         "experiment_set_preinit".to_string(),
@@ -143,9 +127,6 @@ fn test_experiments_recording_before_glean_inits() {
     );
     set_experiment_inactive("experiment_preinit_disabled".to_string());
 
-    let dir = tempfile::tempdir().unwrap();
-    let tmpname = dir.path().display().to_string();
-
     test_reset_glean(
         Configuration {
             data_path: tmpname,
@@ -153,14 +134,14 @@ fn test_experiments_recording_before_glean_inits() {
             upload_enabled: true,
             max_events: None,
             delay_ping_lifetime_io: false,
-            channel: Some("testing".into()),
             server_endpoint: Some("invalid-test-host".into()),
             uploader: None,
+            use_core_mps: false,
+            trim_data_to_registered_pings: false,
         },
         ClientInfoMetrics::unknown(),
         false,
     );
-    crate::block_on_dispatcher();
 
     assert!(test_is_experiment_active(
         "experiment_set_preinit".to_string()
@@ -199,13 +180,13 @@ fn sending_of_foreground_background_pings() {
             _headers: Vec<(String, String)>,
         ) -> net::UploadResult {
             self.sender.send(url).unwrap();
-            net::UploadResult::HttpStatus(200)
+            net::UploadResult::http_status(200)
         }
     }
 
     // Create a custom configuration to use a fake uploader.
     let dir = tempfile::tempdir().unwrap();
-    let tmpname = dir.path().display().to_string();
+    let tmpname = dir.path().to_path_buf();
 
     let cfg = Configuration {
         data_path: tmpname,
@@ -213,13 +194,13 @@ fn sending_of_foreground_background_pings() {
         upload_enabled: true,
         max_events: None,
         delay_ping_lifetime_io: false,
-        channel: Some("testing".into()),
         server_endpoint: Some("invalid-test-host".into()),
         uploader: Some(Box::new(FakeUploader { sender: s })),
+        use_core_mps: false,
+        trim_data_to_registered_pings: false,
     };
 
     let _t = new_glean(Some(cfg), true);
-    crate::block_on_dispatcher();
 
     // Simulate becoming active.
     handle_client_active();
@@ -256,13 +237,11 @@ fn sending_of_foreground_background_pings() {
 fn sending_of_startup_baseline_ping() {
     let _lock = lock_test();
 
-    // Create an instance of Glean, wait for init and then flip the dirty
+    // Create an instance of Glean and then flip the dirty
     // bit to true.
     let data_dir = new_glean(None, true);
 
-    crate::block_on_dispatcher();
-
-    with_glean_mut(|glean| glean.set_dirty_flag(true));
+    glean_core::glean_set_dirty_flag(true);
 
     // Restart glean and wait for a baseline ping to be generated.
     let (s, r) = crossbeam_channel::bounded::<String>(1);
@@ -281,12 +260,12 @@ fn sending_of_startup_baseline_ping() {
             _headers: Vec<(String, String)>,
         ) -> net::UploadResult {
             self.sender.send(url).unwrap();
-            net::UploadResult::HttpStatus(200)
+            net::UploadResult::http_status(200)
         }
     }
 
     // Create a custom configuration to use a fake uploader.
-    let tmpname = data_dir.path().display().to_string();
+    let tmpname = data_dir.path().to_path_buf();
 
     // Now reset Glean: it should still send a baseline ping with reason
     // dirty_startup when starting, because of the dirty bit being set.
@@ -297,9 +276,10 @@ fn sending_of_startup_baseline_ping() {
             upload_enabled: true,
             max_events: None,
             delay_ping_lifetime_io: false,
-            channel: Some("testing".into()),
             server_endpoint: Some("invalid-test-host".into()),
             uploader: Some(Box::new(FakeUploader { sender: s })),
+            use_core_mps: false,
+            trim_data_to_registered_pings: false,
         },
         ClientInfoMetrics::unknown(),
         false,
@@ -307,7 +287,65 @@ fn sending_of_startup_baseline_ping() {
 
     // Wait for the ping to arrive.
     let url = r.recv().unwrap();
-    assert_eq!(url.contains("baseline"), true);
+    assert!(url.contains("baseline"));
+}
+
+#[test]
+fn no_dirty_baseline_on_clean_shutdowns() {
+    let _lock = lock_test();
+
+    // Create an instance of Glean, wait for init and then flip the dirty
+    // bit to true.
+    let data_dir = new_glean(None, true);
+
+    glean_core::glean_set_dirty_flag(true);
+
+    crate::shutdown();
+
+    // Restart glean and wait for a baseline ping to be generated.
+    let (s, r) = crossbeam_channel::bounded::<String>(1);
+
+    // Define a fake uploader that reports back the submission URL
+    // using a crossbeam channel.
+    #[derive(Debug)]
+    pub struct FakeUploader {
+        sender: crossbeam_channel::Sender<String>,
+    }
+    impl net::PingUploader for FakeUploader {
+        fn upload(
+            &self,
+            url: String,
+            _body: Vec<u8>,
+            _headers: Vec<(String, String)>,
+        ) -> net::UploadResult {
+            self.sender.send(url).unwrap();
+            net::UploadResult::http_status(200)
+        }
+    }
+
+    // Create a custom configuration to use a fake uploader.
+    let tmpname = data_dir.path().to_path_buf();
+
+    // Now reset Glean: it should not send a baseline ping, because
+    // we cleared the dirty bit.
+    test_reset_glean(
+        Configuration {
+            data_path: tmpname,
+            application_id: GLOBAL_APPLICATION_ID.into(),
+            upload_enabled: true,
+            max_events: None,
+            delay_ping_lifetime_io: false,
+            server_endpoint: Some("invalid-test-host".into()),
+            uploader: Some(Box::new(FakeUploader { sender: s })),
+            use_core_mps: false,
+            trim_data_to_registered_pings: false,
+        },
+        ClientInfoMetrics::unknown(),
+        false,
+    );
+
+    // We don't expect a startup ping.
+    assert_eq!(r.try_recv(), Err(crossbeam_channel::TryRecvError::Empty));
 }
 
 #[test]
@@ -315,39 +353,37 @@ fn initialize_must_not_crash_if_data_dir_is_messed_up() {
     let _lock = lock_test();
 
     let dir = tempfile::tempdir().unwrap();
-    let tmpdirname = dir.path().display().to_string();
+    let tmpdirname = dir.path();
     // Create a file in the temporary dir and use that as the
     // name of the Glean data dir.
-    let file_path = PathBuf::from(tmpdirname).join("notadir");
+    let file_path = tmpdirname.to_path_buf().join("notadir");
     std::fs::write(file_path.clone(), "test").expect("The test Glean dir file must be created");
 
     let cfg = Configuration {
-        data_path: file_path.to_string_lossy().to_string(),
+        data_path: file_path,
         application_id: GLOBAL_APPLICATION_ID.into(),
         upload_enabled: true,
         max_events: None,
         delay_ping_lifetime_io: false,
-        channel: Some("testing".into()),
         server_endpoint: Some("invalid-test-host".into()),
         uploader: None,
+        use_core_mps: false,
+        trim_data_to_registered_pings: false,
     };
 
     test_reset_glean(cfg, ClientInfoMetrics::unknown(), false);
-    // TODO(bug 1675215): ensure initialize runs through dispatcher.
-    // Glean init is async and, for this test, it bails out early due to
-    // an caused by not being able to create the data dir: we can do nothing
-    // but wait. Tests in other bindings use the dispatcher's test mode, which
-    // runs tasks sequentially on the main thread, so no sleep is required,
-    // because we're guaranteed that, once we reach this point, the full
-    // init potentially ran.
-    std::thread::sleep(std::time::Duration::from_secs(3));
+
+    // We don't need to sleep here.
+    // The `test_reset_glean` already waited on the initialize task.
 }
 
 #[test]
 fn queued_recorded_metrics_correctly_record_during_init() {
     let _lock = lock_test();
+    let dir = tempfile::tempdir().unwrap();
+    let tmpname = dir.path().to_path_buf();
 
-    destroy_glean(true);
+    destroy_glean(true, &tmpname);
 
     let metric = CounterMetric::new(CommonMetricData {
         name: "counter_metric".into(),
@@ -369,7 +405,18 @@ fn queued_recorded_metrics_correctly_record_during_init() {
 
     // Calling `new_glean` here will cause Glean to be initialized and should cause the queued
     // tasks recording metrics to execute
-    let _t = new_glean(None, false);
+    let cfg = Configuration {
+        data_path: tmpname,
+        application_id: GLOBAL_APPLICATION_ID.into(),
+        upload_enabled: true,
+        max_events: None,
+        delay_ping_lifetime_io: false,
+        server_endpoint: Some("invalid-test-host".into()),
+        uploader: None,
+        use_core_mps: false,
+        trim_data_to_registered_pings: false,
+    };
+    let _t = new_glean(Some(cfg), false);
 
     // Verify that the callback was executed by testing for the correct value
     assert!(metric.test_get_value(None).is_some(), "Value must exist");
@@ -381,7 +428,7 @@ fn initializing_twice_is_a_noop() {
     let _lock = lock_test();
 
     let dir = tempfile::tempdir().unwrap();
-    let tmpname = dir.path().display().to_string();
+    let tmpname = dir.path().to_path_buf();
 
     test_reset_glean(
         Configuration {
@@ -390,37 +437,39 @@ fn initializing_twice_is_a_noop() {
             upload_enabled: true,
             max_events: None,
             delay_ping_lifetime_io: false,
-            channel: Some("testing".into()),
             server_endpoint: Some("invalid-test-host".into()),
             uploader: None,
+            use_core_mps: false,
+            trim_data_to_registered_pings: false,
         },
         ClientInfoMetrics::unknown(),
         true,
     );
 
-    crate::block_on_dispatcher();
+    // Glean was initialized and it waited for a full initialization to finish.
+    // We now just want to try to initialize again.
+    // This will bail out early.
 
-    test_reset_glean(
+    crate::initialize(
         Configuration {
             data_path: tmpname,
             application_id: GLOBAL_APPLICATION_ID.into(),
             upload_enabled: true,
             max_events: None,
             delay_ping_lifetime_io: false,
-            channel: Some("testing".into()),
             server_endpoint: Some("invalid-test-host".into()),
             uploader: None,
+            use_core_mps: false,
+            trim_data_to_registered_pings: false,
         },
         ClientInfoMetrics::unknown(),
-        false,
     );
 
-    // TODO(bug 1675215): ensure initialize runs through dispatcher.
-    // Glean init is async and, for this test, it bails out early due to
-    // being initialized: we can do nothing but wait. Tests in other bindings use
-    // the dispatcher's test mode, which runs tasks sequentially on the main
-    // thread, so no sleep is required. Bug 1675215 might fix this, as well.
-    std::thread::sleep(std::time::Duration::from_secs(3));
+    // We don't need to sleep here.
+    // The `test_reset_glean` already waited on the initialize task,
+    // and the 2nd initialize will bail out early.
+    //
+    // All we tested is that this didn't crash.
 }
 
 #[test]
@@ -429,15 +478,51 @@ fn dont_handle_events_when_uninitialized() {
     todo!()
 }
 
+// TODO: Should probably move into glean-core.
 #[test]
 fn the_app_channel_must_be_correctly_set_if_requested() {
     let _lock = lock_test();
 
     let dir = tempfile::tempdir().unwrap();
-    let tmpname = dir.path().display().to_string();
+    let tmpname = dir.path().to_path_buf();
 
-    // No appChannel must be set if nothing was provided through the config
-    // options.
+    // Internal metric, replicated here for testing.
+    let app_channel = StringMetric::new(CommonMetricData {
+        name: "app_channel".into(),
+        category: "".into(),
+        send_in_pings: vec!["glean_client_info".into()],
+        lifetime: Lifetime::Application,
+        disabled: false,
+        ..Default::default()
+    });
+
+    // No app_channel reported.
+    let client_info = ClientInfoMetrics {
+        channel: None,
+        ..ClientInfoMetrics::unknown()
+    };
+    test_reset_glean(
+        Configuration {
+            data_path: tmpname.clone(),
+            application_id: GLOBAL_APPLICATION_ID.into(),
+            upload_enabled: true,
+            max_events: None,
+            delay_ping_lifetime_io: false,
+            server_endpoint: Some("invalid-test-host".into()),
+            uploader: None,
+            use_core_mps: false,
+            trim_data_to_registered_pings: false,
+        },
+        client_info,
+        true,
+    );
+    assert!(app_channel.test_get_value(None).is_none());
+
+    // Custom app_channel reported.
+    let client_info = ClientInfoMetrics {
+        channel: Some("testing".into()),
+        ..ClientInfoMetrics::unknown()
+    };
     test_reset_glean(
         Configuration {
             data_path: tmpname,
@@ -445,26 +530,15 @@ fn the_app_channel_must_be_correctly_set_if_requested() {
             upload_enabled: true,
             max_events: None,
             delay_ping_lifetime_io: false,
-            channel: None,
             server_endpoint: Some("invalid-test-host".into()),
             uploader: None,
+            use_core_mps: false,
+            trim_data_to_registered_pings: false,
         },
-        ClientInfoMetrics::unknown(),
+        client_info,
         true,
     );
-    assert!(core_metrics::internal_metrics::app_channel
-        .test_get_value(None)
-        .is_none());
-
-    // The appChannel must be correctly reported if a channel value
-    // was provided.
-    let _t = new_glean(None, true);
-    assert_eq!(
-        "testing",
-        core_metrics::internal_metrics::app_channel
-            .test_get_value(None)
-            .unwrap()
-    );
+    assert_eq!("testing", app_channel.test_get_value(None).unwrap());
 }
 
 #[test]
@@ -490,39 +564,62 @@ fn basic_metrics_should_be_cleared_when_disabling_uploading() {
 
     assert!(metric.test_get_value(None).is_none());
 
-    metric.set("TEST VALUE");
+    metric.set("TEST VALUE".into());
     assert!(metric.test_get_value(None).is_some());
 
     set_upload_enabled(false);
     assert!(metric.test_get_value(None).is_none());
-    metric.set("TEST VALUE");
+    metric.set("TEST VALUE".into());
     assert!(metric.test_get_value(None).is_none());
 
     set_upload_enabled(true);
     assert!(metric.test_get_value(None).is_none());
-    metric.set("TEST VALUE");
+    metric.set("TEST VALUE".into());
     assert_eq!("TEST VALUE", metric.test_get_value(None).unwrap());
 }
 
+// TODO: Should probably move into glean-core.
 #[test]
 fn core_metrics_should_be_cleared_and_restored_when_disabling_and_enabling_uploading() {
     let _lock = lock_test();
 
-    let _t = new_glean(None, false);
+    let dir = tempfile::tempdir().unwrap();
+    let tmpname = dir.path().to_path_buf();
 
-    assert!(core_metrics::internal_metrics::os_version
-        .test_get_value(None)
-        .is_some());
+    // No app_channel reported.
+    test_reset_glean(
+        Configuration {
+            data_path: tmpname,
+            application_id: GLOBAL_APPLICATION_ID.into(),
+            upload_enabled: true,
+            max_events: None,
+            delay_ping_lifetime_io: false,
+            server_endpoint: Some("invalid-test-host".into()),
+            uploader: None,
+            use_core_mps: false,
+            trim_data_to_registered_pings: false,
+        },
+        ClientInfoMetrics::unknown(),
+        true,
+    );
+
+    // Internal metric, replicated here for testing.
+    let os_version = StringMetric::new(CommonMetricData {
+        name: "os_version".into(),
+        category: "".into(),
+        send_in_pings: vec!["glean_client_info".into()],
+        lifetime: Lifetime::Application,
+        disabled: false,
+        ..Default::default()
+    });
+
+    assert!(os_version.test_get_value(None).is_some());
 
     set_upload_enabled(false);
-    assert!(core_metrics::internal_metrics::os_version
-        .test_get_value(None)
-        .is_none());
+    assert!(os_version.test_get_value(None).is_none());
 
     set_upload_enabled(true);
-    assert!(core_metrics::internal_metrics::os_version
-        .test_get_value(None)
-        .is_some());
+    assert!(os_version.test_get_value(None).is_some());
 }
 
 #[test]
@@ -545,13 +642,13 @@ fn sending_deletion_ping_if_disabled_outside_of_run() {
             _headers: Vec<(String, String)>,
         ) -> net::UploadResult {
             self.sender.send(url).unwrap();
-            net::UploadResult::HttpStatus(200)
+            net::UploadResult::http_status(200)
         }
     }
 
     // Create a custom configuration to use a fake uploader.
     let dir = tempfile::tempdir().unwrap();
-    let tmpname = dir.path().display().to_string();
+    let tmpname = dir.path().to_path_buf();
 
     let cfg = Configuration {
         data_path: tmpname.clone(),
@@ -559,15 +656,14 @@ fn sending_deletion_ping_if_disabled_outside_of_run() {
         upload_enabled: true,
         max_events: None,
         delay_ping_lifetime_io: false,
-        channel: Some("testing".into()),
         server_endpoint: Some("invalid-test-host".into()),
         uploader: None,
+        use_core_mps: false,
+        trim_data_to_registered_pings: false,
     };
 
     let _t = new_glean(Some(cfg), true);
 
-    crate::block_on_dispatcher();
-
     // Now reset Glean and disable upload: it should still send a deletion request
     // ping even though we're just starting.
     test_reset_glean(
@@ -577,9 +673,10 @@ fn sending_deletion_ping_if_disabled_outside_of_run() {
             upload_enabled: false,
             max_events: None,
             delay_ping_lifetime_io: false,
-            channel: Some("testing".into()),
             server_endpoint: Some("invalid-test-host".into()),
             uploader: Some(Box::new(FakeUploader { sender: s })),
+            use_core_mps: false,
+            trim_data_to_registered_pings: false,
         },
         ClientInfoMetrics::unknown(),
         false,
@@ -587,7 +684,7 @@ fn sending_deletion_ping_if_disabled_outside_of_run() {
 
     // Wait for the ping to arrive.
     let url = r.recv().unwrap();
-    assert_eq!(url.contains("deletion-request"), true);
+    assert!(url.contains("deletion-request"));
 }
 
 #[test]
@@ -610,13 +707,13 @@ fn no_sending_of_deletion_ping_if_unchanged_outside_of_run() {
             _headers: Vec<(String, String)>,
         ) -> net::UploadResult {
             self.sender.send(url).unwrap();
-            net::UploadResult::HttpStatus(200)
+            net::UploadResult::http_status(200)
         }
     }
 
     // Create a custom configuration to use a fake uploader.
     let dir = tempfile::tempdir().unwrap();
-    let tmpname = dir.path().display().to_string();
+    let tmpname = dir.path().to_path_buf();
 
     let cfg = Configuration {
         data_path: tmpname.clone(),
@@ -624,15 +721,14 @@ fn no_sending_of_deletion_ping_if_unchanged_outside_of_run() {
         upload_enabled: true,
         max_events: None,
         delay_ping_lifetime_io: false,
-        channel: Some("testing".into()),
         server_endpoint: Some("invalid-test-host".into()),
         uploader: None,
+        use_core_mps: false,
+        trim_data_to_registered_pings: false,
     };
 
     let _t = new_glean(Some(cfg), true);
 
-    crate::block_on_dispatcher();
-
     // Now reset Glean and keep upload enabled: no deletion-request
     // should be sent.
     test_reset_glean(
@@ -642,16 +738,15 @@ fn no_sending_of_deletion_ping_if_unchanged_outside_of_run() {
             upload_enabled: true,
             max_events: None,
             delay_ping_lifetime_io: false,
-            channel: Some("testing".into()),
             server_endpoint: Some("invalid-test-host".into()),
             uploader: Some(Box::new(FakeUploader { sender: s })),
+            use_core_mps: false,
+            trim_data_to_registered_pings: false,
         },
         ClientInfoMetrics::unknown(),
         false,
     );
 
-    crate::block_on_dispatcher();
-
     assert_eq!(0, r.len());
 }
 
@@ -670,9 +765,10 @@ fn test_dirty_flag_is_reset_to_false() {
 #[test]
 fn setting_debug_view_tag_before_initialization_should_not_crash() {
     let _lock = lock_test();
+    let dir = tempfile::tempdir().unwrap();
+    let tmpname = dir.path().to_path_buf();
 
-    destroy_glean(true);
-    assert!(!was_initialize_called());
+    destroy_glean(true, &tmpname);
 
     // Define a fake uploader that reports back the submission headers
     // using a crossbeam channel.
@@ -690,7 +786,7 @@ fn setting_debug_view_tag_before_initialization_should_not_crash() {
             headers: Vec<(String, String)>,
         ) -> net::UploadResult {
             self.sender.send(headers).unwrap();
-            net::UploadResult::HttpStatus(200)
+            net::UploadResult::http_status(200)
         }
     }
 
@@ -698,25 +794,22 @@ fn setting_debug_view_tag_before_initialization_should_not_crash() {
     set_debug_view_tag("valid-tag");
 
     // Create a custom configuration to use a fake uploader.
-    let dir = tempfile::tempdir().unwrap();
-    let tmpname = dir.path().display().to_string();
-
     let cfg = Configuration {
         data_path: tmpname,
         application_id: GLOBAL_APPLICATION_ID.into(),
         upload_enabled: true,
         max_events: None,
         delay_ping_lifetime_io: false,
-        channel: Some("testing".into()),
         server_endpoint: Some("invalid-test-host".into()),
         uploader: Some(Box::new(FakeUploader { sender: s })),
+        use_core_mps: false,
+        trim_data_to_registered_pings: false,
     };
 
     let _t = new_glean(Some(cfg), true);
-    crate::block_on_dispatcher();
 
     // Submit a baseline ping.
-    submit_ping_by_name("baseline", Some("background"));
+    submit_ping_by_name("baseline", Some("inactive"));
 
     // Wait for the ping to arrive.
     let headers = r.recv().unwrap();
@@ -729,9 +822,11 @@ fn setting_debug_view_tag_before_initialization_should_not_crash() {
 #[test]
 fn setting_source_tags_before_initialization_should_not_crash() {
     let _lock = lock_test();
+    let dir = tempfile::tempdir().unwrap();
+    let tmpname = dir.path().to_path_buf();
 
-    destroy_glean(true);
-    assert!(!was_initialize_called());
+    destroy_glean(true, &tmpname);
+    //assert!(!was_initialize_called());
 
     // Define a fake uploader that reports back the submission headers
     // using a crossbeam channel.
@@ -749,16 +844,70 @@ fn setting_source_tags_before_initialization_should_not_crash() {
             headers: Vec<(String, String)>,
         ) -> net::UploadResult {
             self.sender.send(headers).unwrap();
-            net::UploadResult::HttpStatus(200)
+            net::UploadResult::http_status(200)
         }
     }
 
     // Attempt to set source tags before Glean is initialized.
     set_source_tags(vec!["valid-tag1".to_string(), "valid-tag2".to_string()]);
 
+    // Create a custom configuration to use a fake uploader.
+    let cfg = Configuration {
+        data_path: tmpname,
+        application_id: GLOBAL_APPLICATION_ID.into(),
+        upload_enabled: true,
+        max_events: None,
+        delay_ping_lifetime_io: false,
+        server_endpoint: Some("invalid-test-host".into()),
+        uploader: Some(Box::new(FakeUploader { sender: s })),
+        use_core_mps: false,
+        trim_data_to_registered_pings: false,
+    };
+
+    let _t = new_glean(Some(cfg), true);
+
+    // Submit a baseline ping.
+    submit_ping_by_name("baseline", Some("inactive"));
+
+    // Wait for the ping to arrive.
+    let headers = r.recv().unwrap();
+    assert_eq!(
+        "valid-tag1,valid-tag2",
+        headers
+            .iter()
+            .find(|&kv| kv.0 == "X-Source-Tags")
+            .unwrap()
+            .1
+    );
+}
+
+#[test]
+fn setting_source_tags_after_initialization_should_not_crash() {
+    let _lock = lock_test();
+
+    // Define a fake uploader that reports back the submission headers
+    // using a crossbeam channel.
+    let (s, r) = crossbeam_channel::bounded::<Vec<(String, String)>>(1);
+
+    #[derive(Debug)]
+    pub struct FakeUploader {
+        sender: crossbeam_channel::Sender<Vec<(String, String)>>,
+    }
+    impl net::PingUploader for FakeUploader {
+        fn upload(
+            &self,
+            _url: String,
+            _body: Vec<u8>,
+            headers: Vec<(String, String)>,
+        ) -> net::UploadResult {
+            self.sender.send(headers).unwrap();
+            net::UploadResult::http_status(200)
+        }
+    }
+
     // Create a custom configuration to use a fake uploader.
     let dir = tempfile::tempdir().unwrap();
-    let tmpname = dir.path().display().to_string();
+    let tmpname = dir.path().to_path_buf();
 
     let cfg = Configuration {
         data_path: tmpname,
@@ -766,16 +915,21 @@ fn setting_source_tags_before_initialization_should_not_crash() {
         upload_enabled: true,
         max_events: None,
         delay_ping_lifetime_io: false,
-        channel: Some("testing".into()),
         server_endpoint: Some("invalid-test-host".into()),
         uploader: Some(Box::new(FakeUploader { sender: s })),
+        use_core_mps: false,
+        trim_data_to_registered_pings: false,
     };
 
     let _t = new_glean(Some(cfg), true);
-    crate::block_on_dispatcher();
+
+    // Attempt to set source tags after `Glean.initialize` is called,
+    // but before Glean is fully initialized.
+    //assert!(was_initialize_called());
+    set_source_tags(vec!["valid-tag1".to_string(), "valid-tag2".to_string()]);
 
     // Submit a baseline ping.
-    submit_ping_by_name("baseline", Some("background"));
+    submit_ping_by_name("baseline", Some("inactive"));
 
     // Wait for the ping to arrive.
     let headers = r.recv().unwrap();
@@ -818,13 +972,13 @@ fn flipping_upload_enabled_respects_order_of_events() {
             _headers: Vec<(String, String)>,
         ) -> net::UploadResult {
             self.sender.send(url).unwrap();
-            net::UploadResult::HttpStatus(200)
+            net::UploadResult::http_status(200)
         }
     }
 
     // Create a custom configuration to use a fake uploader.
     let dir = tempfile::tempdir().unwrap();
-    let tmpname = dir.path().display().to_string();
+    let tmpname = dir.path().to_path_buf();
 
     let cfg = Configuration {
         data_path: tmpname,
@@ -832,9 +986,10 @@ fn flipping_upload_enabled_respects_order_of_events() {
         upload_enabled: true,
         max_events: None,
         delay_ping_lifetime_io: false,
-        channel: Some("testing".into()),
         server_endpoint: Some("invalid-test-host".into()),
         uploader: Some(Box::new(FakeUploader { sender: s })),
+        use_core_mps: false,
+        trim_data_to_registered_pings: false,
     };
 
     // We create a ping and a metric before we initialize Glean
@@ -854,21 +1009,18 @@ fn flipping_upload_enabled_respects_order_of_events() {
     set_upload_enabled(false);
 
     // Set data and try to submit a custom ping.
-    metric.set("some-test-value");
+    metric.set("some-test-value".into());
     sample_ping.submit(None);
 
     // Wait for the ping to arrive.
     let url = r.recv().unwrap();
-    assert_eq!(url.contains("deletion-request"), true);
+    assert!(url.contains("deletion-request"));
 }
 
 #[test]
 fn registering_pings_before_init_must_work() {
     let _lock = lock_test();
 
-    destroy_glean(true);
-    assert!(!was_initialize_called());
-
     // Define a fake uploader that reports back the submission headers
     // using a crossbeam channel.
     let (s, r) = crossbeam_channel::bounded::<String>(1);
@@ -885,7 +1037,7 @@ fn registering_pings_before_init_must_work() {
             _headers: Vec<(String, String)>,
         ) -> net::UploadResult {
             self.sender.send(url).unwrap();
-            net::UploadResult::HttpStatus(200)
+            net::UploadResult::http_status(200)
         }
     }
 
@@ -894,7 +1046,7 @@ fn registering_pings_before_init_must_work() {
 
     // Create a custom configuration to use a fake uploader.
     let dir = tempfile::tempdir().unwrap();
-    let tmpname = dir.path().display().to_string();
+    let tmpname = dir.path().to_path_buf();
 
     let cfg = Configuration {
         data_path: tmpname,
@@ -902,13 +1054,13 @@ fn registering_pings_before_init_must_work() {
         upload_enabled: true,
         max_events: None,
         delay_ping_lifetime_io: false,
-        channel: Some("testing".into()),
         server_endpoint: Some("invalid-test-host".into()),
         uploader: Some(Box::new(FakeUploader { sender: s })),
+        use_core_mps: false,
+        trim_data_to_registered_pings: false,
     };
 
     let _t = new_glean(Some(cfg), true);
-    crate::block_on_dispatcher();
 
     // Submit a baseline ping.
     sample_ping.submit(None);
@@ -917,3 +1069,175 @@ fn registering_pings_before_init_must_work() {
     let url = r.recv().unwrap();
     assert!(url.contains("pre-register"));
 }
+
+#[test]
+fn test_a_ping_before_submission() {
+    let _lock = lock_test();
+
+    // Define a fake uploader that reports back the submission headers
+    // using a crossbeam channel.
+    let (s, r) = crossbeam_channel::bounded::<String>(1);
+
+    #[derive(Debug)]
+    pub struct FakeUploader {
+        sender: crossbeam_channel::Sender<String>,
+    }
+    impl net::PingUploader for FakeUploader {
+        fn upload(
+            &self,
+            url: String,
+            _body: Vec<u8>,
+            _headers: Vec<(String, String)>,
+        ) -> net::UploadResult {
+            self.sender.send(url).unwrap();
+            net::UploadResult::http_status(200)
+        }
+    }
+
+    // Create a custom configuration to use a fake uploader.
+    let dir = tempfile::tempdir().unwrap();
+    let tmpname = dir.path().to_path_buf();
+
+    let cfg = Configuration {
+        data_path: tmpname,
+        application_id: GLOBAL_APPLICATION_ID.into(),
+        upload_enabled: true,
+        max_events: None,
+        delay_ping_lifetime_io: false,
+        server_endpoint: Some("invalid-test-host".into()),
+        uploader: Some(Box::new(FakeUploader { sender: s })),
+        use_core_mps: false,
+        trim_data_to_registered_pings: false,
+    };
+
+    let _t = new_glean(Some(cfg), true);
+
+    // Create a custom ping and register it.
+    let sample_ping = PingType::new("custom1", true, true, vec![]);
+
+    let metric = CounterMetric::new(CommonMetricData {
+        name: "counter_metric".into(),
+        category: "test".into(),
+        send_in_pings: vec!["custom1".into()],
+        lifetime: Lifetime::Application,
+        disabled: false,
+        dynamic_label: None,
+    });
+
+    metric.add(1);
+
+    sample_ping.test_before_next_submit(move |reason| {
+        assert_eq!(None, reason);
+        assert_eq!(1, metric.test_get_value(None).unwrap());
+    });
+
+    // Submit a baseline ping.
+    sample_ping.submit(None);
+
+    // Wait for the ping to arrive.
+    let url = r.recv().unwrap();
+    assert!(url.contains("custom1"));
+}
+
+#[test]
+fn test_boolean_get_num_errors() {
+    let _lock = lock_test();
+
+    let _t = new_glean(None, false);
+
+    let metric = BooleanMetric::new(CommonMetricData {
+        name: "counter_metric".into(),
+        category: "test".into(),
+        send_in_pings: vec!["custom1".into()],
+        lifetime: Lifetime::Application,
+        disabled: false,
+        dynamic_label: Some(str::to_string("asdf")),
+    });
+
+    // Check specifically for an invalid label
+    let result = metric.test_get_num_recorded_errors(ErrorType::InvalidLabel);
+
+    assert_eq!(result, 0);
+}
+
+#[test]
+fn signaling_done() {
+    let _lock = lock_test();
+
+    // Define a fake uploader that reports back the submission URL
+    // using a crossbeam channel.
+    #[derive(Debug)]
+    pub struct FakeUploader {
+        barrier: Arc<Barrier>,
+        counter: Arc<Mutex<HashMap<ThreadId, u32>>>,
+    }
+    impl net::PingUploader for FakeUploader {
+        fn upload(
+            &self,
+            _url: String,
+            _body: Vec<u8>,
+            _headers: Vec<(String, String)>,
+        ) -> net::UploadResult {
+            let mut map = self.counter.lock().unwrap();
+            *map.entry(thread::current().id()).or_insert(0) += 1;
+
+            // Wait for the sync.
+            self.barrier.wait();
+
+            // Signal that this uploader thread is done.
+            net::UploadResult::done()
+        }
+    }
+
+    // Create a custom configuration to use a fake uploader.
+    let dir = tempfile::tempdir().unwrap();
+    let tmpname = dir.path().to_path_buf();
+
+    // We use a barrier to sync this test thread with the uploader thread.
+    let barrier = Arc::new(Barrier::new(2));
+    // We count how many times `upload` was invoked per thread.
+    let call_count = Arc::new(Mutex::default());
+
+    let cfg = Configuration {
+        data_path: tmpname,
+        application_id: GLOBAL_APPLICATION_ID.into(),
+        upload_enabled: true,
+        max_events: None,
+        delay_ping_lifetime_io: false,
+        server_endpoint: Some("invalid-test-host".into()),
+        uploader: Some(Box::new(FakeUploader {
+            barrier: Arc::clone(&barrier),
+            counter: Arc::clone(&call_count),
+        })),
+        use_core_mps: false,
+        trim_data_to_registered_pings: false,
+    };
+
+    let _t = new_glean(Some(cfg), true);
+
+    // Define a new ping and submit it.
+    const PING_NAME: &str = "test-ping";
+    let custom_ping = private::PingType::new(PING_NAME, true, true, vec![]);
+    custom_ping.submit(None);
+    custom_ping.submit(None);
+
+    // Sync up with the upload thread.
+    barrier.wait();
+
+    // Submit another ping and wait for it to do work.
+    custom_ping.submit(None);
+
+    // Sync up with the upload thread again.
+    // This will not be the same thread as the one before (hopefully).
+    barrier.wait();
+
+    // No one's ever gonna wait for the uploader thread (the RLB doesn't store the handle to it),
+    // so all we can do is hope it finishes within time.
+    std::thread::sleep(std::time::Duration::from_millis(100));
+
+    let map = call_count.lock().unwrap();
+    assert_eq!(2, map.len(), "should have launched 2 uploader threads");
+    for &count in map.values() {
+        assert_eq!(1, count, "each thread should call upload only once");
+    }
+}
