@@ -3,17 +3,19 @@
 // file, You can obtain one at https://mozilla.org/MPL/2.0/.
 
 use std::convert::TryFrom;
+use std::sync::atomic::{AtomicU8, Ordering};
 
 use crate::error::{Error, ErrorKind};
-#[allow(unused_imports)]
-use crate::metrics::{dynamic_label, LabeledMetric};
+use crate::metrics::labeled::validate_dynamic_label;
 use crate::Glean;
+use serde::{Deserialize, Serialize};
 
 /// The supported metrics' lifetimes.
 ///
 /// A metric's lifetime determines when its stored data gets reset.
-#[derive(Copy, Clone, Debug, PartialEq, Eq)]
+#[derive(Copy, Clone, Debug, PartialEq, Eq, Deserialize, Serialize)]
 #[repr(i32)] // Use i32 to be compatible with our JNA definition
+#[serde(rename_all = "lowercase")]
 pub enum Lifetime {
     /// The metric is reset with each sent ping
     Ping,
@@ -54,7 +56,7 @@ impl TryFrom<i32> for Lifetime {
 }
 
 /// The common set of data shared across all different metric types.
-#[derive(Default, Debug, Clone)]
+#[derive(Default, Debug, Clone, Deserialize, Serialize)]
 pub struct CommonMetricData {
     /// The metric's name.
     pub name: String,
@@ -70,25 +72,52 @@ pub struct CommonMetricData {
     pub disabled: bool,
     /// Dynamic label.
     ///
-    /// When a [`LabeledMetric<T>`](LabeledMetric) factory creates the specific
+    /// When a [`LabeledMetric<T>`](crate::metrics::LabeledMetric) factory creates the specific
     /// metric to be recorded to, dynamic labels are stored in the specific
     /// label so that we can validate them when the Glean singleton is
     /// available.
     pub dynamic_label: Option<String>,
 }
 
-impl CommonMetricData {
+#[derive(Default, Debug)]
+pub struct CommonMetricDataInternal {
+    pub inner: CommonMetricData,
+    pub disabled: AtomicU8,
+}
+
+impl Clone for CommonMetricDataInternal {
+    fn clone(&self) -> Self {
+        Self {
+            inner: self.inner.clone(),
+            disabled: AtomicU8::new(self.disabled.load(Ordering::Relaxed)),
+        }
+    }
+}
+
+impl From<CommonMetricData> for CommonMetricDataInternal {
+    fn from(input_data: CommonMetricData) -> Self {
+        Self {
+            inner: input_data.clone(),
+            disabled: AtomicU8::new(u8::from(input_data.disabled)),
+        }
+    }
+}
+
+impl CommonMetricDataInternal {
     /// Creates a new metadata object.
     pub fn new<A: Into<String>, B: Into<String>, C: Into<String>>(
         category: A,
         name: B,
         ping_name: C,
-    ) -> CommonMetricData {
-        CommonMetricData {
-            name: name.into(),
-            category: category.into(),
-            send_in_pings: vec![ping_name.into()],
-            ..Default::default()
+    ) -> CommonMetricDataInternal {
+        CommonMetricDataInternal {
+            inner: CommonMetricData {
+                name: name.into(),
+                category: category.into(),
+                send_in_pings: vec![ping_name.into()],
+                ..Default::default()
+            },
+            disabled: AtomicU8::new(0),
         }
     }
 
@@ -97,10 +126,10 @@ impl CommonMetricData {
     /// If `category` is empty, it's ommitted.
     /// Otherwise, it's the combination of the metric's `category` and `name`.
     pub(crate) fn base_identifier(&self) -> String {
-        if self.category.is_empty() {
-            self.name.clone()
+        if self.inner.category.is_empty() {
+            self.inner.name.clone()
         } else {
-            format!("{}.{}", self.category, self.name)
+            format!("{}.{}", self.inner.category, self.inner.name)
         }
     }
 
@@ -111,20 +140,15 @@ impl CommonMetricData {
     pub(crate) fn identifier(&self, glean: &Glean) -> String {
         let base_identifier = self.base_identifier();
 
-        if let Some(label) = &self.dynamic_label {
-            dynamic_label(glean, self, &base_identifier, label)
+        if let Some(label) = &self.inner.dynamic_label {
+            validate_dynamic_label(glean, self, &base_identifier, label)
         } else {
             base_identifier
         }
     }
 
-    /// Whether this metric should be recorded.
-    pub fn should_record(&self) -> bool {
-        !self.disabled
-    }
-
     /// The list of storages this metric should be recorded into.
     pub fn storage_names(&self) -> &[String] {
-        &self.send_in_pings
+        &self.inner.send_in_pings
     }
 }
