@@ -10,158 +10,144 @@ use std::path::Path;
 use std::str;
 use std::sync::RwLock;
 
+use rkv::migrator::Migrator;
 use rkv::StoreOptions;
 
-// Select the LMDB-powered storage backend when the feature is not activated.
-#[cfg(not(feature = "rkv-safe-mode"))]
-mod backend {
-    use std::path::Path;
+/// Unwrap a `Result`s `Ok` value or do the specified action.
+///
+/// This is an alternative to the question-mark operator (`?`),
+/// when the other action should not be to return the error.
+macro_rules! unwrap_or {
+    ($expr:expr, $or:expr) => {
+        match $expr {
+            Ok(x) => x,
+            Err(_) => {
+                $or;
+            }
+        }
+    };
+}
 
-    /// cbindgen:ignore
-    pub type Rkv = rkv::Rkv<rkv::backend::LmdbEnvironment>;
-    /// cbindgen:ignore
-    pub type SingleStore = rkv::SingleStore<rkv::backend::LmdbDatabase>;
-    /// cbindgen:ignore
-    pub type Writer<'t> = rkv::Writer<rkv::backend::LmdbRwTransaction<'t>>;
+/// cbindgen:ignore
+pub type Rkv = rkv::Rkv<rkv::backend::SafeModeEnvironment>;
+/// cbindgen:ignore
+pub type SingleStore = rkv::SingleStore<rkv::backend::SafeModeDatabase>;
+/// cbindgen:ignore
+pub type Writer<'t> = rkv::Writer<rkv::backend::SafeModeRwTransaction<'t>>;
 
-    pub fn rkv_new(path: &Path) -> Result<Rkv, rkv::StoreError> {
-        Rkv::new::<rkv::backend::Lmdb>(path)
-    }
-
-    /// No migration necessary when staying with LMDB.
-    pub fn migrate(_path: &Path, _dst_env: &Rkv) {
-        // Intentionally left empty.
+pub fn rkv_new(path: &Path) -> std::result::Result<Rkv, rkv::StoreError> {
+    match Rkv::new::<rkv::backend::SafeMode>(path) {
+        // An invalid file can mean:
+        // 1. An empty file.
+        // 2. A corrupted file.
+        //
+        // In both instances there's not much we can do.
+        // Drop the data by removing the file, and start over.
+        Err(rkv::StoreError::FileInvalid) => {
+            let safebin = path.join("data.safe.bin");
+            fs::remove_file(safebin).map_err(|_| rkv::StoreError::FileInvalid)?;
+            // Now try again, we only handle that error once.
+            Rkv::new::<rkv::backend::SafeMode>(path)
+        }
+        other => other,
     }
 }
 
-// Select the "safe mode" storage backend when the feature is activated.
-#[cfg(feature = "rkv-safe-mode")]
-mod backend {
-    use rkv::migrator::Migrator;
-    use std::{fs, path::Path};
-
-    /// cbindgen:ignore
-    pub type Rkv = rkv::Rkv<rkv::backend::SafeModeEnvironment>;
-    /// cbindgen:ignore
-    pub type SingleStore = rkv::SingleStore<rkv::backend::SafeModeDatabase>;
-    /// cbindgen:ignore
-    pub type Writer<'t> = rkv::Writer<rkv::backend::SafeModeRwTransaction<'t>>;
-
-    pub fn rkv_new(path: &Path) -> Result<Rkv, rkv::StoreError> {
-        match Rkv::new::<rkv::backend::SafeMode>(path) {
-            // An invalid file can mean:
-            // 1. An empty file.
-            // 2. A corrupted file.
-            //
-            // In both instances there's not much we can do.
-            // Drop the data by removing the file, and start over.
-            Err(rkv::StoreError::FileInvalid) => {
-                let safebin = path.join("data.safe.bin");
-                fs::remove_file(safebin).map_err(|_| rkv::StoreError::FileInvalid)?;
-                // Now try again, we only handle that error once.
-                Rkv::new::<rkv::backend::SafeMode>(path)
+fn delete_and_log(path: &Path, msg: &str) {
+    if let Err(err) = fs::remove_file(path) {
+        match err.kind() {
+            std::io::ErrorKind::NotFound => {
+                // Silently drop this error, the file was already non-existing.
             }
-            other => other,
+            _ => log::warn!("{}", msg),
         }
     }
-
-    fn delete_and_log(path: &Path, msg: &str) {
-        if let Err(err) = fs::remove_file(path) {
-            match err.kind() {
-                std::io::ErrorKind::NotFound => {
-                    // Silently drop this error, the file was already non-existing.
-                }
-                _ => log::warn!("{}", msg),
-            }
-        }
-    }
-
-    fn delete_lmdb_database(path: &Path) {
-        let datamdb = path.join("data.mdb");
-        delete_and_log(&datamdb, "Failed to delete old data.");
-
-        let lockmdb = path.join("lock.mdb");
-        delete_and_log(&lockmdb, "Failed to delete old lock.");
-    }
-
-    /// Migrate from LMDB storage to safe-mode storage.
-    ///
-    /// This migrates the data once, then deletes the LMDB storage.
-    /// The safe-mode storage must be empty for it to work.
-    /// Existing data will not be overwritten.
-    /// If the destination database is not empty the LMDB database is deleted
-    /// without migrating data.
-    /// This is a no-op if no LMDB database file exists.
-    pub fn migrate(path: &Path, dst_env: &Rkv) {
-        use rkv::{MigrateError, StoreError};
-
-        log::debug!("Migrating files in {}", path.display());
-
-        // Shortcut if no data to migrate is around.
-        let datamdb = path.join("data.mdb");
-        if !datamdb.exists() {
-            log::debug!("No data to migrate.");
-            return;
-        }
-
-        // We're handling the same error cases as `easy_migrate_lmdb_to_safe_mode`,
-        // but annotate each why they don't cause problems for Glean.
-        // Additionally for known cases we delete the LMDB database regardless.
-        let should_delete =
-            match Migrator::open_and_migrate_lmdb_to_safe_mode(path, |builder| builder, dst_env) {
-                // Source environment is corrupted.
-                // We start fresh with the new database.
-                Err(MigrateError::StoreError(StoreError::FileInvalid)) => true,
-                Err(MigrateError::StoreError(StoreError::DatabaseCorrupted)) => true,
-                // Path not accessible.
-                // Somehow our directory vanished between us creating it and reading from it.
-                // Nothing we can do really.
-                Err(MigrateError::StoreError(StoreError::IoError(_))) => true,
-                // Path accessible but incompatible for configuration.
-                // This should not happen, we never used storages that safe-mode doesn't understand.
-                // If it does happen, let's start fresh and use the safe-mode from now on.
-                Err(MigrateError::StoreError(StoreError::UnsuitableEnvironmentPath(_))) => true,
-                // Nothing to migrate.
-                // Source database was empty. We just start fresh anyway.
-                Err(MigrateError::SourceEmpty) => true,
-                // Migrating would overwrite.
-                // Either a previous migration failed and we still started writing data,
-                // or someone placed back an old data file.
-                // In any case we better stay on the new data and delete the old one.
-                Err(MigrateError::DestinationNotEmpty) => {
-                    log::warn!("Failed to migrate old data. Destination was not empty");
-                    true
-                }
-                // An internal lock was poisoned.
-                // This would only happen if multiple things run concurrently and one crashes.
-                Err(MigrateError::ManagerPoisonError) => false,
-                // Couldn't close source environment and delete files on disk (e.g. other stores still open).
-                // This could only happen if multiple instances are running,
-                // we leave files in place.
-                Err(MigrateError::CloseError(_)) => false,
-                // Other store errors are never returned from the migrator.
-                // We need to handle them to please rustc.
-                Err(MigrateError::StoreError(_)) => false,
-                // Other errors can't happen, so this leaves us with the Ok case.
-                // This already deleted the LMDB files.
-                Ok(()) => false,
-            };
-
-        if should_delete {
-            log::debug!("Need to delete remaining LMDB files.");
-            delete_lmdb_database(path);
-        }
-
-        log::debug!("Migration ended. Safe-mode database in {}", path.display());
-    }
 }
 
+fn delete_lmdb_database(path: &Path) {
+    let datamdb = path.join("data.mdb");
+    delete_and_log(&datamdb, "Failed to delete old data.");
+
+    let lockmdb = path.join("lock.mdb");
+    delete_and_log(&lockmdb, "Failed to delete old lock.");
+}
+
+/// Migrate from LMDB storage to safe-mode storage.
+///
+/// This migrates the data once, then deletes the LMDB storage.
+/// The safe-mode storage must be empty for it to work.
+/// Existing data will not be overwritten.
+/// If the destination database is not empty the LMDB database is deleted
+/// without migrating data.
+/// This is a no-op if no LMDB database file exists.
+pub fn migrate(path: &Path, dst_env: &Rkv) {
+    use rkv::{MigrateError, StoreError};
+
+    log::debug!("Migrating files in {}", path.display());
+
+    // Shortcut if no data to migrate is around.
+    let datamdb = path.join("data.mdb");
+    if !datamdb.exists() {
+        log::debug!("No data to migrate.");
+        return;
+    }
+
+    // We're handling the same error cases as `easy_migrate_lmdb_to_safe_mode`,
+    // but annotate each why they don't cause problems for Glean.
+    // Additionally for known cases we delete the LMDB database regardless.
+    let should_delete =
+        match Migrator::open_and_migrate_lmdb_to_safe_mode(path, |builder| builder, dst_env) {
+            // Source environment is corrupted.
+            // We start fresh with the new database.
+            Err(MigrateError::StoreError(StoreError::FileInvalid)) => true,
+            Err(MigrateError::StoreError(StoreError::DatabaseCorrupted)) => true,
+            // Path not accessible.
+            // Somehow our directory vanished between us creating it and reading from it.
+            // Nothing we can do really.
+            Err(MigrateError::StoreError(StoreError::IoError(_))) => true,
+            // Path accessible but incompatible for configuration.
+            // This should not happen, we never used storages that safe-mode doesn't understand.
+            // If it does happen, let's start fresh and use the safe-mode from now on.
+            Err(MigrateError::StoreError(StoreError::UnsuitableEnvironmentPath(_))) => true,
+            // Nothing to migrate.
+            // Source database was empty. We just start fresh anyway.
+            Err(MigrateError::SourceEmpty) => true,
+            // Migrating would overwrite.
+            // Either a previous migration failed and we still started writing data,
+            // or someone placed back an old data file.
+            // In any case we better stay on the new data and delete the old one.
+            Err(MigrateError::DestinationNotEmpty) => {
+                log::warn!("Failed to migrate old data. Destination was not empty");
+                true
+            }
+            // An internal lock was poisoned.
+            // This would only happen if multiple things run concurrently and one crashes.
+            Err(MigrateError::ManagerPoisonError) => false,
+            // Couldn't close source environment and delete files on disk (e.g. other stores still open).
+            // This could only happen if multiple instances are running,
+            // we leave files in place.
+            Err(MigrateError::CloseError(_)) => false,
+            // Other store errors are never returned from the migrator.
+            // We need to handle them to please rustc.
+            Err(MigrateError::StoreError(_)) => false,
+            // Other errors can't happen, so this leaves us with the Ok case.
+            // This already deleted the LMDB files.
+            Ok(()) => false,
+        };
+
+    if should_delete {
+        log::debug!("Need to delete remaining LMDB files.");
+        delete_lmdb_database(path);
+    }
+
+    log::debug!("Migration ended. Safe-mode database in {}", path.display());
+}
+
+use crate::common_metric_data::CommonMetricDataInternal;
 use crate::metrics::Metric;
-use crate::CommonMetricData;
 use crate::Glean;
 use crate::Lifetime;
 use crate::Result;
-use backend::*;
 
 pub struct Database {
     /// Handle to the database environment.
@@ -235,21 +221,6 @@ impl Database {
     /// It also loads any Lifetime::Ping data that might be
     /// persisted, in case `delay_ping_lifetime_io` is set.
     pub fn new(data_path: &Path, delay_ping_lifetime_io: bool) -> Result<Self> {
-        #[cfg(all(windows, not(feature = "rkv-safe-mode")))]
-        {
-            // The underlying lmdb wrapper implementation
-            // cannot actually handle non-UTF8 paths on Windows.
-            // It will unconditionally panic if passed one.
-            // See
-            // https://github.com/mozilla/lmdb-rs/blob/df1c2f56e3088f097c719c57b9925ab51e26f3f4/src/environment.rs#L43-L53
-            //
-            // To avoid this, in case we're using LMDB on Windows (that's just testing now though),
-            // we simply error out earlier.
-            if data_path.to_str().is_none() {
-                return Err(crate::Error::utf8_error());
-            }
-        }
-
         let path = data_path.join("db");
         log::debug!("Database path: {:?}", path.display());
         let file_size = database_size(&path);
@@ -294,7 +265,7 @@ impl Database {
 
     /// Creates the storage directories and inits rkv.
     fn open_rkv(path: &Path) -> Result<Rkv> {
-        fs::create_dir_all(&path)?;
+        fs::create_dir_all(path)?;
 
         let rkv = rkv_new(path)?;
         migrate(path, &rkv);
@@ -478,7 +449,7 @@ impl Database {
     }
 
     /// Records a metric in the underlying storage system.
-    pub fn record(&self, glean: &Glean, data: &CommonMetricData, value: &Metric) {
+    pub fn record(&self, glean: &Glean, data: &CommonMetricDataInternal, value: &Metric) {
         // If upload is disabled we don't want to record.
         if !glean.is_upload_enabled() {
             return;
@@ -487,7 +458,7 @@ impl Database {
         let name = data.identifier(glean);
 
         for ping_name in data.storage_names() {
-            if let Err(e) = self.record_per_lifetime(data.lifetime, ping_name, &name, value) {
+            if let Err(e) = self.record_per_lifetime(data.inner.lifetime, ping_name, &name, value) {
                 log::error!("Failed to record metric into {}: {:?}", ping_name, e);
             }
         }
@@ -537,7 +508,7 @@ impl Database {
 
     /// Records the provided value, with the given lifetime,
     /// after applying a transformation function.
-    pub fn record_with<F>(&self, glean: &Glean, data: &CommonMetricData, mut transform: F)
+    pub fn record_with<F>(&self, glean: &Glean, data: &CommonMetricDataInternal, mut transform: F)
     where
         F: FnMut(Option<Metric>) -> Metric,
     {
@@ -549,7 +520,7 @@ impl Database {
         let name = data.identifier(glean);
         for ping_name in data.storage_names() {
             if let Err(e) =
-                self.record_per_lifetime_with(data.lifetime, ping_name, &name, &mut transform)
+                self.record_per_lifetime_with(data.inner.lifetime, ping_name, &name, &mut transform)
             {
                 log::error!("Failed to record metric into {}: {:?}", ping_name, e);
             }
@@ -643,16 +614,16 @@ impl Database {
             ping_lifetime_data
                 .write()
                 .expect("Can't access ping lifetime data as writable")
-                .clear();
+                .retain(|metric_id, _| !metric_id.starts_with(storage_name));
         }
 
         self.write_with_store(Lifetime::Ping, |mut writer, store| {
             let mut metrics = Vec::new();
             {
-                let mut iter = store.iter_from(&writer, &storage_name)?;
+                let mut iter = store.iter_from(&writer, storage_name)?;
                 while let Some(Ok((metric_id, _))) = iter.next() {
                     if let Ok(metric_id) = std::str::from_utf8(metric_id) {
-                        if !metric_id.starts_with(&storage_name) {
+                        if !metric_id.starts_with(storage_name) {
                             break;
                         }
                         metrics.push(metric_id.to_owned());
@@ -782,7 +753,7 @@ impl Database {
                     // There is no need for `get_storage_key` here because
                     // the key is already formatted from when it was saved
                     // to ping_lifetime_data.
-                    store.put(&mut writer, &key, &rkv::Value::Blob(&encoded))?;
+                    store.put(&mut writer, key, &rkv::Value::Blob(&encoded))?;
                 }
                 writer.commit()?;
                 Ok(())
@@ -796,7 +767,6 @@ impl Database {
 mod test {
     use super::*;
     use crate::tests::new_glean;
-    use crate::CommonMetricData;
     use std::collections::HashMap;
     use std::path::Path;
     use tempfile::tempdir;
@@ -824,25 +794,12 @@ mod test {
 
         let res = Database::new(&path, false);
 
-        #[cfg(feature = "rkv-safe-mode")]
-        {
-            assert!(
-                res.is_ok(),
-                "Database should succeed at {}: {:?}",
-                path.display(),
-                res
-            );
-        }
-
-        #[cfg(not(feature = "rkv-safe-mode"))]
-        {
-            assert!(
-                res.is_err(),
-                "Database should fail at {}: {:?}",
-                path.display(),
-                res
-            );
-        }
+        assert!(
+            res.is_ok(),
+            "Database should succeed at {}: {:?}",
+            path.display(),
+            res
+        );
     }
 
     #[test]
@@ -1321,6 +1278,71 @@ mod test {
         }
     }
 
+    #[test]
+    fn test_delayed_ping_lifetime_clear() {
+        // Init the database in a temporary directory.
+        let dir = tempdir().unwrap();
+        let db = Database::new(dir.path(), true).unwrap();
+        let test_storage = "test-storage";
+
+        assert!(db.ping_lifetime_data.is_some());
+
+        // Attempt to record a known value.
+        let test_value1 = "test-value1";
+        let test_metric_id1 = "telemetry_test.test_name1";
+        db.record_per_lifetime(
+            Lifetime::Ping,
+            test_storage,
+            test_metric_id1,
+            &Metric::String(test_value1.to_string()),
+        )
+        .unwrap();
+
+        {
+            let data = match &db.ping_lifetime_data {
+                Some(ping_lifetime_data) => ping_lifetime_data,
+                None => panic!("Expected `ping_lifetime_data` to exist here!"),
+            };
+            let data = data.read().unwrap();
+            // Verify that test_value1 is in memory.
+            assert!(data
+                .get(&format!("{}#{}", test_storage, test_metric_id1))
+                .is_some());
+        }
+
+        // Clear ping lifetime storage for a storage that isn't test_storage.
+        // Doesn't matter what it's called, just that it isn't test_storage.
+        db.clear_ping_lifetime_storage(&(test_storage.to_owned() + "x"))
+            .unwrap();
+
+        {
+            let data = match &db.ping_lifetime_data {
+                Some(ping_lifetime_data) => ping_lifetime_data,
+                None => panic!("Expected `ping_lifetime_data` to exist here!"),
+            };
+            let data = data.read().unwrap();
+            // Verify that test_value1 is still in memory.
+            assert!(data
+                .get(&format!("{}#{}", test_storage, test_metric_id1))
+                .is_some());
+        }
+
+        // Clear test_storage's ping lifetime storage.
+        db.clear_ping_lifetime_storage(test_storage).unwrap();
+
+        {
+            let data = match &db.ping_lifetime_data {
+                Some(ping_lifetime_data) => ping_lifetime_data,
+                None => panic!("Expected `ping_lifetime_data` to exist here!"),
+            };
+            let data = data.read().unwrap();
+            // Verify that test_value1 is no longer in memory.
+            assert!(data
+                .get(&format!("{}#{}", test_storage, test_metric_id1))
+                .is_none());
+        }
+    }
+
     #[test]
     fn doesnt_record_when_upload_is_disabled() {
         let (mut glean, dir) = new_glean(None);
@@ -1328,7 +1350,7 @@ mod test {
         // Init the database in a temporary directory.
 
         let test_storage = "test-storage";
-        let test_data = CommonMetricData::new("category", "name", test_storage);
+        let test_data = CommonMetricDataInternal::new("category", "name", test_storage);
         let test_metric_id = test_data.identifier(&glean);
 
         // Attempt to record metric with the record and record_with functions,
@@ -1411,27 +1433,6 @@ mod test {
         );
     }
 
-    /// LDMB ignores an empty database file just fine.
-    #[cfg(not(feature = "rkv-safe-mode"))]
-    #[test]
-    fn empty_data_file() {
-        let dir = tempdir().unwrap();
-
-        // Create database directory structure.
-        let database_dir = dir.path().join("db");
-        fs::create_dir_all(&database_dir).expect("create database dir");
-
-        // Create empty database file.
-        let datamdb = database_dir.join("data.mdb");
-        let f = fs::File::create(datamdb).expect("create database file");
-        drop(f);
-
-        Database::new(dir.path(), false).unwrap();
-
-        assert!(dir.path().exists());
-    }
-
-    #[cfg(feature = "rkv-safe-mode")]
     mod safe_mode {
         use std::fs::File;
 
